library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library UNISIM;
use UNISIM.VComponents.all;

entity Testgenerator16Bit is
    Port ( 
		STOP_GENERATOR : in STD_LOGIC;
		DATA_DATA 		: out STD_LOGIC_VECTOR(15 downto 0);
		DATA_VALID		: out STD_LOGIC;

		Reset 			: in STD_LOGIC;
		Clk 				: in  STD_LOGIC);
		
end Testgenerator16Bit;

architecture Behavioral of Testgenerator16Bit is
        type STATE_Type is (
				SM_RESET,
				SM_ENTRAINMENT_0_1,
				SM_ENTRAINMENT_2_3,
				SM_ENTRAINMENT_4_5,
				SM_ENTRAINMENT_6_7,
				SM_ENTRAINMENT_8_9,
				SM_ENTRAINMENT_10_11,
            SM_INIT_TRANSFER_A_B,
            SM_ADDRESS_01,
				SM_BYTE_02_03,
				SM_BYTE_04_05,
				SM_BYTE_06_07,
				SM_BYTE_08_09,
				SM_BYTE_10_11,
				SM_BYTE_12_13,
				SM_BYTE_14_15,
				SM_BYTE_16_17,
				SM_BYTE_18_19,
				SM_BYTE_20_21,
				SM_BYTE_22_23,
				SM_BYTE_24_25,
				SM_BYTE_26_27,
				SM_BYTE_28_29,
				SM_BYTE_30_31,
				SM_BYTE_32_33,
				SM_BYTE_34_35,
				SM_BYTE_36_37,
				SM_BYTE_38_39,
				SM_BYTE_40_41,
				SM_BYTE_42_43,
				SM_BYTE_44_45,
				SM_BYTE_46_47,
				SM_BYTE_48_49,
            SM_BYTE_50_51,
            SM_BYTE_52_53,
            SM_COOLDOWN
        );
        signal SPIMACHINE_STATE : STATE_TYPE;

        constant ENTRAINMENT_0          : STD_LOGIC_VECTOR(7 downto 0) := "10101010";
        constant ENTRAINMENT_1          : STD_LOGIC_VECTOR(7 downto 0) := "11001010";
        constant ENTRAINMENT_2          : STD_LOGIC_VECTOR(7 downto 0) := "11010101";
        constant ENTRAINMENT_3          : STD_LOGIC_VECTOR(7 downto 0) := "01001010";
        constant ENTRAINMENT_4          : STD_LOGIC_VECTOR(7 downto 0) := "10101001";
        constant ENTRAINMENT_5          : STD_LOGIC_VECTOR(7 downto 0) := "10011100";
        constant ENTRAINMENT_6          : STD_LOGIC_VECTOR(7 downto 0) := "11100011";
        constant ENTRAINMENT_7          : STD_LOGIC_VECTOR(7 downto 0) := "10101111";
        constant ENTRAINMENT_8          : STD_LOGIC_VECTOR(7 downto 0) := "00101010";
        constant ENTRAINMENT_9          : STD_LOGIC_VECTOR(7 downto 0) := "00000000";
		  constant ENTRAINMENT_10         : STD_LOGIC_VECTOR(7 downto 0) := "00000000";
		  constant ENTRAINMENT_11         : STD_LOGIC_VECTOR(7 downto 0) := "00000000";
        constant INIT_TRANSFER_A        : STD_LOGIC_VECTOR(7 downto 0) := "11111111";
		  constant INIT_TRANSFER_B        : STD_LOGIC_VECTOR(7 downto 0) := "10101010";

		  signal Counter 						 : unsigned (15 downto 0) := (others => '0');
begin

        TestGeneratorProcess: process(Reset,Clk)
        begin
			if (Reset = '1') then
				SPIMACHINE_STATE <= SM_RESET;
				DATA_DATA <= (others => '0');
				DATA_VALID <= '0';	
				Counter <= (others => '0');
				
         elsif (rising_edge(Clk)) then
			
				-- Default:
				DATA_DATA <= (others => '0');
				DATA_VALID <= '0';
				Counter <= Counter;
				
				case SPIMACHINE_STATE is
					when SM_RESET =>
						SPIMACHINE_STATE <= SM_ENTRAINMENT_0_1;
						
					when SM_ENTRAINMENT_0_1 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_ENTRAINMENT_0_1;
						else
							SPIMACHINE_STATE <= SM_ENTRAINMENT_2_3;
							DATA_VALID <= '1';
							DATA_DATA <= ENTRAINMENT_0 & ENTRAINMENT_1;
						end if;
						
					when SM_ENTRAINMENT_2_3 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_ENTRAINMENT_2_3;
						else
							SPIMACHINE_STATE <= SM_ENTRAINMENT_4_5;
							DATA_VALID <= '1';
							DATA_DATA <= ENTRAINMENT_2 & ENTRAINMENT_3;
						end if;
						
					when SM_ENTRAINMENT_4_5 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_ENTRAINMENT_4_5;
						else
							SPIMACHINE_STATE <= SM_ENTRAINMENT_6_7;
							DATA_VALID <= '1';
							DATA_DATA <= ENTRAINMENT_4 & ENTRAINMENT_5;
						end if;
						
					when SM_ENTRAINMENT_6_7 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_ENTRAINMENT_6_7;
						else
							SPIMACHINE_STATE <= SM_ENTRAINMENT_8_9;
							DATA_VALID <= '1';
							DATA_DATA <= ENTRAINMENT_6 & ENTRAINMENT_7;
						end if;
						
					when SM_ENTRAINMENT_8_9 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_ENTRAINMENT_8_9;
						else
							SPIMACHINE_STATE <= SM_ENTRAINMENT_10_11;
							DATA_VALID <= '1';
							DATA_DATA <= ENTRAINMENT_8 & ENTRAINMENT_9;
						end if;
						
					when SM_ENTRAINMENT_10_11 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_ENTRAINMENT_10_11;
						else
							SPIMACHINE_STATE <= SM_INIT_TRANSFER_A_B;
							DATA_VALID <= '1';
							DATA_DATA <= ENTRAINMENT_10 & ENTRAINMENT_11;
						end if;
						
            	when SM_INIT_TRANSFER_A_B =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_INIT_TRANSFER_A_B;
						else
							SPIMACHINE_STATE <= SM_ADDRESS_01;
							DATA_VALID <= '1';
							DATA_DATA <= INIT_TRANSFER_A & INIT_TRANSFER_B;
						end if;
						
            	when SM_ADDRESS_01 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_ADDRESS_01;
						else
							SPIMACHINE_STATE <= SM_BYTE_02_03;
							DATA_VALID <= '1';
							DATA_DATA <= "00000001" & "11111111";
						end if;

					
					when SM_BYTE_02_03 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_02_03;
						else
							SPIMACHINE_STATE <= SM_BYTE_04_05;
							DATA_VALID <= '1';
							DATA_DATA <= "11111111" & "11111111";
						end if;
						
					when SM_BYTE_04_05 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_04_05;
						else
							SPIMACHINE_STATE <= SM_BYTE_06_07;
							DATA_VALID <= '1';
							DATA_DATA <= "11111111" & "00000000";
						end if;
						
					when SM_BYTE_06_07 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_06_07;
						else
							SPIMACHINE_STATE <= SM_BYTE_08_09;
							DATA_VALID <= '1';
							DATA_DATA <= "00000000" & "00000000";
						end if;
						
					when SM_BYTE_08_09 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_08_09;
						else
							SPIMACHINE_STATE <= SM_BYTE_10_11;
							DATA_VALID <= '1';
							DATA_DATA <= "00000000" & "00000000";
						end if;
						
					when SM_BYTE_10_11 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_10_11;
						else
							SPIMACHINE_STATE <= SM_BYTE_12_13;
							DATA_VALID <= '1';
							DATA_DATA <= std_logic_vector(Counter(7 downto 0)) & std_logic_vector(Counter(15 downto 8));
						end if;
						
					when SM_BYTE_12_13 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_12_13;
						else
							SPIMACHINE_STATE <= SM_BYTE_14_15;
							DATA_VALID <= '1';
							DATA_DATA <= std_logic_vector(Counter(7 downto 0)) & std_logic_vector(Counter(15 downto 8));
						end if;
						
					when SM_BYTE_14_15 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_14_15;
						else
							SPIMACHINE_STATE <= SM_BYTE_16_17;
							DATA_VALID <= '1';
							DATA_DATA <= std_logic_vector(Counter(7 downto 0)) & std_logic_vector(Counter(15 downto 8));
						end if;
						
					when SM_BYTE_16_17 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_16_17;
						else
							SPIMACHINE_STATE <= SM_BYTE_18_19;
							DATA_VALID <= '1';
							DATA_DATA <= std_logic_vector(Counter(7 downto 0)) & std_logic_vector(Counter(15 downto 8));
						end if;
						
					when SM_BYTE_18_19 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_18_19;
						else
							SPIMACHINE_STATE <= SM_BYTE_20_21;
							DATA_VALID <= '1';
							DATA_DATA <= std_logic_vector(Counter(7 downto 0)) & std_logic_vector(Counter(15 downto 8));
						end if;
						
					when SM_BYTE_20_21 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_20_21;
						else
							SPIMACHINE_STATE <= SM_BYTE_22_23;
							DATA_VALID <= '1';
							DATA_DATA <= std_logic_vector(Counter(7 downto 0)) & std_logic_vector(Counter(15 downto 8));
						end if;
						
					when SM_BYTE_22_23 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_22_23;
						else
							SPIMACHINE_STATE <= SM_BYTE_24_25;
							DATA_VALID <= '1';
							DATA_DATA <= std_logic_vector(Counter(7 downto 0)) & std_logic_vector(Counter(15 downto 8));
						end if;
						
					when SM_BYTE_24_25 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_24_25;
						else
							SPIMACHINE_STATE <= SM_BYTE_26_27;
							DATA_VALID <= '1';
							DATA_DATA <= std_logic_vector(Counter(7 downto 0)) & std_logic_vector(Counter(15 downto 8));
						end if;
						
					when SM_BYTE_26_27 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_26_27;
						else
							SPIMACHINE_STATE <= SM_BYTE_28_29;
							DATA_VALID <= '1';
							DATA_DATA <= std_logic_vector(Counter(7 downto 0)) & std_logic_vector(Counter(15 downto 8));
						end if;
						
					when SM_BYTE_28_29 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_28_29;
						else
							SPIMACHINE_STATE <= SM_BYTE_30_31;
							DATA_VALID <= '1';
							DATA_DATA <= std_logic_vector(Counter(7 downto 0)) & std_logic_vector(Counter(15 downto 8));
						end if;
						
					when SM_BYTE_30_31 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_30_31;
						else
							SPIMACHINE_STATE <= SM_BYTE_32_33;
							DATA_VALID <= '1';
							DATA_DATA <= std_logic_vector(Counter(7 downto 0)) & std_logic_vector(Counter(15 downto 8));
						end if;
						
						
					when SM_BYTE_32_33 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_32_33;
						else
							SPIMACHINE_STATE <= SM_BYTE_34_35;
							DATA_VALID <= '1';
							DATA_DATA <= std_logic_vector(Counter(7 downto 0)) & std_logic_vector(Counter(15 downto 8));
						end if;
						
					when SM_BYTE_34_35 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_34_35;
						else
							SPIMACHINE_STATE <= SM_BYTE_36_37;
							DATA_VALID <= '1';
							DATA_DATA <= std_logic_vector(Counter(7 downto 0)) & std_logic_vector(Counter(15 downto 8));
						end if;
						
					when SM_BYTE_36_37 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_36_37;
						else
							SPIMACHINE_STATE <= SM_BYTE_38_39;
							DATA_VALID <= '1';
							DATA_DATA <= std_logic_vector(Counter(7 downto 0)) & std_logic_vector(Counter(15 downto 8));
						end if;
						
					when SM_BYTE_38_39 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_38_39;
						else
							SPIMACHINE_STATE <= SM_BYTE_40_41;
							DATA_VALID <= '1';
							DATA_DATA <= std_logic_vector(Counter(7 downto 0)) & std_logic_vector(Counter(15 downto 8));
						end if;
						
					when SM_BYTE_40_41 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_40_41;
						else
							SPIMACHINE_STATE <= SM_BYTE_42_43;
							DATA_VALID <= '1';
							DATA_DATA <= std_logic_vector(Counter(7 downto 0)) & std_logic_vector(Counter(15 downto 8));
						end if;
						
					when SM_BYTE_42_43 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_42_43;
						else
							SPIMACHINE_STATE <= SM_BYTE_44_45;
							DATA_VALID <= '1';
							DATA_DATA <= "11111111" & "11111111";
						end if;
						
					when SM_BYTE_44_45 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_44_45;
						else
							SPIMACHINE_STATE <= SM_BYTE_46_47;
							DATA_VALID <= '1';
							DATA_DATA <= "11111111" & "11111111";
						end if;
						
					when SM_BYTE_46_47 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_46_47;
						else
							SPIMACHINE_STATE <= SM_BYTE_48_49;
							DATA_VALID <= '1';
							DATA_DATA <= "11111111" & "11111111";
						end if;
						
					when SM_BYTE_48_49 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_48_49;
						else
							SPIMACHINE_STATE <= SM_BYTE_50_51;
							DATA_VALID <= '1';
							DATA_DATA <= "11111111" & "11111111";
						end if;
						
            	when SM_BYTE_50_51 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_50_51;
						else
							SPIMACHINE_STATE <= SM_BYTE_52_53;
							DATA_VALID <= '1';
							DATA_DATA <= "11111111" & "11111111";
						end if;
						
            	when SM_BYTE_52_53 =>
						if STOP_GENERATOR = '1' then
							SPIMACHINE_STATE <= SM_BYTE_52_53;
						else
							SPIMACHINE_STATE <= SM_COOLDOWN;
							DATA_VALID <= '1';
							DATA_DATA <= "00000000" & "00000000";
						end if;
				
            	when SM_COOLDOWN	=>				
						SPIMACHINE_STATE <= SM_INIT_TRANSFER_A_B;
						Counter <= Counter + 1;
				
				end case;
			end if;
        end process TestGeneratorProcess;


end Behavioral;


