library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity IGLOO_TOP_UUT is
end IGLOO_TOP_UUT;

architecture IGLOO_TOP_UUT_ARCH of IGLOO_TOP_UUT is

  signal SPI_ZL_CS           : STD_LOGIC := '0';
  signal SPI_SDI             : STD_LOGIC := '0';
  signal SPI_SDO             : STD_LOGIC := '0';
  signal SPI_CLK             : STD_LOGIC := '0';

  signal RHD_MOSI            : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
  signal RHD_MISO            : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
  signal RHD_CLK             : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
  signal RHD_CS              : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');


  signal DEBUG_STATE         : std_logic_vector(8 downto 0) := (others => '0');
  signal DEBUG_SELECTOR_STATE: std_logic_vector(4 downto 0) := (others => '0');

  signal DEBUG_CLOCK         : std_logic := '0';
  signal DEBUG_SELECTOR_CLOCK: std_logic_vector(1 downto 0) := (others => '0');

  signal Reset               :  STD_LOGIC := '0';
  signal Clk                 :  STD_LOGIC := '0';

  -- Aux Signals
  signal MasterCounter : unsigned(127 downto 0) := (others=>'0');
  signal InputCounter : unsigned(127 downto 0) := (others=>'0');
  -- First bit is not used!!!
  signal TEST_SPI_OUTPUT : std_logic_vector (120 downto 0) := (others => '0');
  constant TEST_SPI_DUMMY : std_logic_vector (7 downto 0) := "00000000";
  constant TEST_SPI_DUMMY_II : std_logic_vector (104 downto 0) := (others => '0');

  signal SPI_CLK_LAST : std_logic;
  signal SPI_ZL_CS_LAST : std_logic;

  -- Reverse required...
  constant HeaderByte_1	: std_logic_vector(7 downto 0) := "00001100"; -- "00110000"; -- 48
  constant HeaderByte_2	: std_logic_vector(7 downto 0) := "10010110"; -- "01101001"; -- 105
  constant Implant_ID     : std_logic_vector(7 downto 0) := "10101100"; -- "00110101"; -- Implant ID 53
  constant Command_Enable : std_logic_vector(7 downto 0) := "10100110"; -- "01100101"; -- 101 e
  constant Command_Disable: std_logic_vector(7 downto 0) := "00100110"; -- "01100100"; -- 100 d
  constant Command_Channel: std_logic_vector(7 downto 0) := "11000110"; -- "01100011"; --  99 c
  constant Command_Filter : std_logic_vector(7 downto 0) := "01100110"; -- "01100110"; -- 102 f
  constant Command_SRate  : std_logic_vector(7 downto 0) := "11001110"; -- "01110011"; -- 115 s
  constant Command_ADCReso: std_logic_vector(7 downto 0) := "01001110"; -- "01110010"; -- 114 r
  constant Command_ADCResT: std_logic_vector(7 downto 0) := "00101110"; -- "01110100"; -- 116 t

BEGIN
  SPI_SDO <= TEST_SPI_OUTPUT(0);

  MYImplant : entity work.IGLOO_TOP
    port map (
      SPI_ZL_CS => SPI_ZL_CS,
      SPI_SDI => SPI_SDI,
      SPI_SDO => SPI_SDO,
      SPI_CLK => SPI_CLK,
      RHD_MOSI => RHD_MOSI,
      RHD_MISO => RHD_MISO,
      RHD_CLK  => RHD_CLK,
      RHD_CS  => RHD_CS,
      DEBUG_STATE => DEBUG_STATE,
      DEBUG_SELECTOR_STATE => DEBUG_SELECTOR_STATE,
      DEBUG_CLOCK => DEBUG_CLOCK,
      DEBUG_SELECTOR_CLOCK => DEBUG_SELECTOR_CLOCK,
      Reset => Reset,
      Clk => Clk
  );

  TheMasterClock : process
  begin
    clockloop: for i in 0 to 200000 loop
      Clk <= NOT Clk;
      wait for 21 ns;
    end loop clockloop;
    wait;
  end process TheMasterClock;


  -- For the test the parameters in ZARLINK_CONNECT_INIT are set to:
  -- constant Zarlink_WaitForReset : unsigned(23 downto 0) := to_unsigned(12,24);
  -- constant Zarlink_Retry : unsigned(16 downto 0) := to_unsigned(7,17);

  --			TEST_SPI_OUTPUT <=   "00000000" & "00000000" & "00000000"
  --									 & "00000000" & "00000000" & "00000000"
  --									 & "00000000" & "00000000" & "00000000"
  --									 & "00000000" & "00000000" & "00000000"
  --									 & "00000000" & "00000000" & TEST_SPI_DUMMY;

  TheSecondClock : process(Clk)
  begin
    if rising_edge(Clk) then

      if MasterCounter < to_unsigned(20,128) then
        Reset <= '0';

      else
        Reset <= '1';
      end if;

      SPI_CLK_LAST <= SPI_CLK;
      SPI_ZL_CS_LAST <= SPI_ZL_CS;
      -- Test Connection Initiation:
      -- [->]
      -- Check if connection is exisiting (shoudn't be...)
      -- "0000000001001101"
      -- No Connection
      if (SPI_ZL_CS_LAST = '1') AND (SPI_ZL_CS = '0') AND (InputCounter = to_unsigned(0,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- we will se for a Zarlink connection init:
        -- WAIT
        -- "0100101011111010" ZC_INITIATE_RESET
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(1,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- WAIT
        -- "0000011011010100" Set SPI Speed
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(2,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "1111000001011000" Disable Watchdog
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(3,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "0000011011010100" Set SPI Speed
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(4,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "0000000001001101" Check IRQ_RADIOREADY
        -- Let the first IRQ RADIOREADY fail and then answer correctly
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(5,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(6,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00010000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "1111000011101000" ZC_REG_MAC_MODUSER
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(7,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "1010000010011000" ZC_REG_MAC_CHANNEL
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(8,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "0111000000100000" ZC_REG_REG_TXBUFFER_BSIZE_A
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(9,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "0111100001000000" ZC_REG_REG_RXBUFFER_BSIZE_A
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(10,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "1111100010100000" ZC_REG_TXBUFF_MAXPACKSIZE_A
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(11,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "0100000001000100" ZC_ALLOW_HOUSEKEEPINGMESSAGES
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(12,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "0101010101000010" ZC_IMD_COPY_ID_A
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(13,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "0101010111000010" ZC_IMD_COPY_ID_B
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(14,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "0101010100100010" ZC_IMD_COPY_ID_C
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(15,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "0101010111001000" ZC_IMD_ID_A
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(16,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "0101010100101000" ZC_IMD_ID_B
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(17,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "0101010110101000" ZC_IMD_ID_C
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(18,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "0000000001010100" ZC_REG_MAC_INITCOM_A_IMD
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(19,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- "0000000001001101" ZC_WAIT_IRQ_LINKREADY_A_REQ
        -- Let the first IRQ LINKREADY fail and then answer correctly
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(20,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(21,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00100000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- Yes Connection
        -- [2] = '1'
        -- Everything is still fine... (LINK READY)
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(22,128)) then
        -- Yes Connection
        -- [2] = '1'
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "00100000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- Test Connection Initiation
        -- [<-] CHECKED AND OK!!! (DON't touch ZarlinkConnectionInit and CheckConnection anymore)
        -- 6961
        -- Now the system checks the ingoing buffer
        -- if there is nothing it checks the Linkready
        -- and if this fails then we start all over again ...
        -- Thus we have one thing in the incoming buffer
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(23,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- [GET INGOING BUFFER FILL STATUS] CHECKED AND OK!
        -- 7300
        -- "0000000010110101" Read the data
        -- Set Filter to "1111"
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(24,128)) then
        TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000"
        & "00000000" & "00000000" & "00000000"
        & "00000000" & "00000000" & "00000000"
        & "11110000" & Command_Filter & Implant_ID
        & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- OK
        -- Sample Rate
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(25,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;

      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(26,128)) then
        TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000"
        & "00000000" & "00000000" & "00000000"
        & "00000000" & "00000000" & "00000000"
        & "10000000" & Command_SRate & Implant_ID
        & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;

        -- OK
        -- Cut away higher bits
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(27,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(28,128)) then
        TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000"
        & "00000000" & "00000000" & "00000000"
        & "00000000" & "00000000" & "00000000"
        & "10010000" & Command_ADCReso & Implant_ID
        & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;

        -- OK
        -- Cut away lower bits
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(29,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(30,128)) then
        TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000"
        & "00000000" & "00000000" & "00000000"
        & "00000000" & "00000000" & "00000000"
        & "11000000" & Command_ADCResT & Implant_ID
        & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- OK
        -- ChannelMask
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(31,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(32,128)) then
        TEST_SPI_OUTPUT <= "0" & "11101010" & "01010101" & "01010101"
        & "01010101" & "10101010" & "10101010"
        & "10101010" & "10101010" & "10101010"
        & "10101011" & Command_Channel & Implant_ID
        & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;

      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(33,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(34,128)) then
        TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000"
        & "00000000" & "00000000" & "00000000"
        & "00000000" & "00000000" & "11111010"
        & "10101010" & "10101010" & "10101010"
        & "10101010" & "11110101" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- OK
        -- Enable
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(35,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(36,128)) then
        TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000"
        & "00000000" & "00000000" & "00000000"
        & "00000000" & "00000000" & "00000000"
        & "00000000" & Command_Enable & Implant_ID
        & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- OK
        -- Disable
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(37,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(38,128)) then
        TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000"
        & "00000000" & "00000000" & "00000000"
        & "00000000" & "00000000" & "00000000"
        & "00000000" & Command_Disable & Implant_ID
        & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- OK
        -- Enable
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(39,128)) then
        TEST_SPI_OUTPUT <= TEST_SPI_DUMMY_II & "10000000" & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
      elsif (SPI_ZL_CS_LAST = '0') AND (SPI_ZL_CS = '1') AND (InputCounter = to_unsigned(40,128)) then
        TEST_SPI_OUTPUT <= "0" & "00000000" & "00000000" & "00000000"
        & "00000000" & "00000000" & "00000000"
        & "00000000" & "00000000" & "00000000"
        & "00000000" & Command_Enable & Implant_ID
        & HeaderByte_2 & HeaderByte_1 & TEST_SPI_DUMMY;
        InputCounter <= InputCounter + 1;
        -- OK

      else
        if (SPI_CLK = '0') AND (SPI_CLK_LAST = '1') then
          TEST_SPI_OUTPUT <= "0" & TEST_SPI_OUTPUT(120 downto 1);
        else
          TEST_SPI_OUTPUT <= TEST_SPI_OUTPUT;
        end if;
      end if;

      MasterCounter <= MasterCounter + 1;
    end if;
  end process TheSecondClock;


end architecture IGLOO_TOP_UUT_ARCH;
