library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity AD5360_PHY is
    Port ( 
		-- Input
		Value 	: in  STD_LOGIC_VECTOR (23 downto 0);
		Valid  	: in  STD_LOGIC;
		
		-- Signal to be ready
		Busy		: out STD_LOGIC;
		
		-- Output
		MOSI			: out STD_LOGIC;
		SCLK			: out STD_LOGIC;
		CS				: out STD_LOGIC;

		-- Usual ports 
		Reset			: in  STD_LOGIC;
		Clk 			: in  STD_LOGIC);
end AD5360_PHY;

architecture Behavioral of AD5360_PHY is

	signal SCLK_GATE 				: STD_LOGIC := '0';
    signal SCLK_GATE_DELAYED		: STD_LOGIC := '0';
	signal DATABUFFER 			    : STD_LOGIC_VECTOR (23 downto 0) := (others => '0');

	-- The states of the state machine
	type STATE_Type is (
		SM_RESET,
		SM_IDLE,
		SM_UP_CS,
		SM_DOWN_CS,
		SM_BIT_23,
		SM_BIT_22,
		SM_BIT_21,
		SM_BIT_20,
		SM_BIT_19,
		SM_BIT_18,
		SM_BIT_17,
		SM_BIT_16,
		SM_BIT_15,
		SM_BIT_14,
		SM_BIT_13,
		SM_BIT_12,
		SM_BIT_11,
		SM_BIT_10,		
		SM_BIT_9,				
		SM_BIT_8,				
		SM_BIT_7,				
		SM_BIT_6,				
		SM_BIT_5,				
		SM_BIT_4,				
		SM_BIT_3,				
		SM_BIT_2,				
		SM_BIT_1,				
		SM_BIT_0,
		SM_Cooldown
	);	
	signal SPIMACHINE_STATE : STATE_TYPE;

begin

	-- We only enable the SCLK when we need it...
	SCLKGATE: process(Reset,Clk,SCLK_GATE_DELAYED)
	begin
		if (Reset = '1') then
			SCLK <= '0';
		else
			if SCLK_GATE_DELAYED = '1' then
				SCLK <= Clk;
			else
				SCLK <= '0';
			end if;
		
		end if;
	end process SCLKGATE;

	-- We pump out the data via SPI
	SPIFSM : process(Reset,Clk)
	begin
		if (Reset = '1') then
			SPIMACHINE_STATE <= SM_RESET;
			SCLK_GATE <= '0';
            SCLK_GATE_DELAYED <= '0';
			CS <= '1';
			MOSI <= '0';
			DATABUFFER <= (others => '0');
			Busy <= '1';
			
		elsif (rising_edge(Clk)) then
			-- Default 
			SCLK_GATE <= '1';
            SCLK_GATE_DELAYED <= SCLK_GATE;
			SPIMACHINE_STATE <= SM_RESET;
			CS <= '0';
			MOSI <= '0';
			DATABUFFER <= DATABUFFER;
			Busy <= '1';
			
			case SPIMACHINE_STATE is
				when SM_RESET =>
					SPIMACHINE_STATE <= SM_IDLE;
					SCLK_GATE <= '0';
					CS <= '1';
					
				when SM_IDLE =>
					if Valid = '1' then
						SPIMACHINE_STATE <= SM_UP_CS;					
					else
						SPIMACHINE_STATE <= SM_IDLE;					
						Busy <= '0';
					end if;
					SCLK_GATE <= '0';
					CS <= '1';
					DATABUFFER <= Value;
					
				when SM_UP_CS =>
					SPIMACHINE_STATE <= SM_DOWN_CS;
					SCLK_GATE <= '0';
					CS <= '1';
					
				when SM_DOWN_CS =>
					SPIMACHINE_STATE <= SM_BIT_23;
					-- SCLK_GATE <= '0';
					SCLK_GATE <= '1';
					
				when SM_BIT_23 =>
					SPIMACHINE_STATE <= SM_BIT_22;
					MOSI <= DATABUFFER(23);
				
				when SM_BIT_22 =>
					SPIMACHINE_STATE <= SM_BIT_21;
					MOSI <= DATABUFFER(22);
					
				when SM_BIT_21 =>
					SPIMACHINE_STATE <= SM_BIT_20;
					MOSI <= DATABUFFER(21);
					
				when SM_BIT_20 =>
					SPIMACHINE_STATE <= SM_BIT_19;
					MOSI <= DATABUFFER(20);
					
				when SM_BIT_19 =>
					SPIMACHINE_STATE <= SM_BIT_18;
					MOSI <= DATABUFFER(19);
				
				when SM_BIT_18 =>
					SPIMACHINE_STATE <= SM_BIT_17;
					MOSI <= DATABUFFER(18);
					
				when SM_BIT_17 =>
					SPIMACHINE_STATE <= SM_BIT_16;
					MOSI <= DATABUFFER(17);
					
				when SM_BIT_16 =>
					SPIMACHINE_STATE <= SM_BIT_15;
					MOSI <= DATABUFFER(16);
					
				when SM_BIT_15 =>
					SPIMACHINE_STATE <= SM_BIT_14;
					MOSI <= DATABUFFER(15);
					
				when SM_BIT_14 =>
					SPIMACHINE_STATE <= SM_BIT_13;
					MOSI <= DATABUFFER(14);
					
				when SM_BIT_13 =>
					SPIMACHINE_STATE <= SM_BIT_12;
					MOSI <= DATABUFFER(13);
					
				when SM_BIT_12 =>
					SPIMACHINE_STATE <= SM_BIT_11;
					MOSI <= DATABUFFER(12);
					
				when SM_BIT_11 =>
					SPIMACHINE_STATE <= SM_BIT_10;
					MOSI <= DATABUFFER(11);
					
				when SM_BIT_10 =>
					SPIMACHINE_STATE <= SM_BIT_9;
					MOSI <= DATABUFFER(10);
					
				when SM_BIT_9 =>
					SPIMACHINE_STATE <= SM_BIT_8;
					MOSI <= DATABUFFER(9);
					
				when SM_BIT_8 =>
					SPIMACHINE_STATE <= SM_BIT_7;
					MOSI <= DATABUFFER(8);
					
				when SM_BIT_7 =>
					SPIMACHINE_STATE <= SM_BIT_6;
					MOSI <= DATABUFFER(7);
					
				when SM_BIT_6 =>
					SPIMACHINE_STATE <= SM_BIT_5;
					MOSI <= DATABUFFER(6);
					
				when SM_BIT_5 =>
					SPIMACHINE_STATE <= SM_BIT_4;
					MOSI <= DATABUFFER(5);
					
				when SM_BIT_4 =>
					SPIMACHINE_STATE <= SM_BIT_3;
					MOSI <= DATABUFFER(4);
					
				when SM_BIT_3 =>
					SPIMACHINE_STATE <= SM_BIT_2;
					MOSI <= DATABUFFER(3);
					
				when SM_BIT_2 =>
					SPIMACHINE_STATE <= SM_BIT_1;
					MOSI <= DATABUFFER(2);
					
				when SM_BIT_1 =>
					SPIMACHINE_STATE <= SM_BIT_0;
					MOSI <= DATABUFFER(1);
					
				when SM_BIT_0 =>
					SPIMACHINE_STATE <= SM_Cooldown;
					MOSI <= DATABUFFER(0);
					SCLK_GATE <= '0';
					
				when SM_Cooldown =>
					SPIMACHINE_STATE <= SM_IDLE;
					SCLK_GATE <= '0';
					
				--when others => 
					--SPIMACHINE_STATE <= SM_RESET;
					--SCLK_GATE <= '0';
					--CS <= '1';
			end case;
		end if;
	
	end process SPIFSM;
	
end Behavioral;

