library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;


entity AD5360Programmer is
    Port ( 

				-- The Data for programming the DAC 
				Controller_ChannelID 			        : in  STD_LOGIC_VECTOR(3 downto 0);
				Controller_Value					    : in  STD_LOGIC_VECTOR(15 downto 0);
				Controller_Valid					    : in  STD_LOGIC;
				Controller_SetMode				        : in  STD_LOGIC_VECTOR(1 downto 0);
				PauseCounter						    : in  STD_LOGIC_VECTOR(39 downto 0);
				
				Controller_Wait					        : out	STD_LOGIC;
				Controller_SPIACTION				    : in  STD_LOGIC;
				
				-- SPI and Co
				NOT_LDAC		    					: out STD_LOGIC; 
				NOT_RESET_DAC	    				    : out STD_LOGIC; 
				MOSI									: out STD_LOGIC;
				CS										: out STD_LOGIC;
				SCLK									: out STD_LOGIC;

                UpdateSwitch                            : out STD_LOGIC;
			
				Clk 									: in  STD_LOGIC;
                Reset 			    				    : in  STD_LOGIC
	 );
end AD5360Programmer;

architecture Behavioral of AD5360Programmer is

	signal Controller_Busy			: STD_LOGIC := '0';

-- Buffer Input
   signal PauseCounter_Output	: unsigned(39 downto 0) := (others => '0');	
	
	type STATE_Type is (
		SM_RESET,
		SM_PAUSE,
		SM_INITALIZE,
		SM_WRITE,
		SM_LDAC,
      SM_COOLDOWN,
      SM_WAIT
	);	
	signal SPIMACHINE_STATE : STATE_TYPE;	


		-- Start SPI Transmission 
	signal SPI_START  				: STD_LOGIC := '0';
	signal SPI_START_LAST			: STD_LOGIC := '0';
	
	-- Wait until the reset of the FPGA
	signal ResetCounter 				: unsigned (39 downto 0) := (others => '0');
	constant ResetCounter_END		    : unsigned (39 downto 0) := to_unsigned(12000000,40);
    
    
    -- Needs to be larger then 30; 50 is working. The interval ]30,50[ is not tested yet.
    -- constant COOLDOWN_COUNTER		    : unsigned (39 downto 0) := to_unsigned(50,40);
    -- I tried to get a clean 40KSamples per second 
    -- constant COOLDOWN_COUNTER		    : unsigned (39 downto 0) := to_unsigned(83,40); -- 24MHZ
    constant COOLDOWN_COUNTER		    : unsigned (39 downto 0) := to_unsigned(107,40); -- 25MHZ

	-- Use for the simulation:
	-- constant ResetCounter_END		: unsigned (39 downto 0) := to_unsigned(12,40);	

begin


MyController: entity work.AD5360_Controller
    Port MAP( 
			ChannelID 	    	    => Controller_ChannelID,
			Value					=> Controller_Value,
			Valid					=> Controller_Valid,
			SetMode		    	    => Controller_SetMode,
			
			StartWrite 	    	    => SPI_START,
			Busy_EXTERNAL		    => Controller_Busy,

            UpdateSwitch            => UpdateSwitch,
			
	 		MOSI					=> MOSI,
			SCLK					=> SCLK,
			CS						=> CS,
			Clk 					=> Clk,
         Reset 		   	=> Reset
		);


	DAC_FSM : process(Reset,Clk)
	begin
		if (Reset = '1') then
			SPIMACHINE_STATE <= SM_RESET;
			SPI_START_LAST <= '0';
			SPI_START <= '0';
			ResetCounter <= (others => '0');
			NOT_RESET_DAC <= '0';
			NOT_LDAC <= '1';
            PauseCounter_Output <= (others => '0');
            Controller_Wait <= '1';
			
		elsif (rising_edge(Clk)) then
			-- Default 
			SPIMACHINE_STATE <= SM_RESET;
			SPI_START_LAST <= SPI_START;
			SPI_START <= '0';
			ResetCounter <= ResetCounter + 1;
			NOT_RESET_DAC <= '1';
			NOT_LDAC <= '1';
            PauseCounter_Output <= PauseCounter_Output;
            Controller_Wait <= '1';
			
			case SPIMACHINE_STATE is
				when SM_RESET =>	
					SPIMACHINE_STATE <= SM_PAUSE;
					ResetCounter <= (others => '0');
					
				when SM_PAUSE => 
					if ResetCounter = ResetCounter_END then
						SPIMACHINE_STATE <= SM_INITALIZE;
					else
						SPIMACHINE_STATE <= SM_PAUSE;
					end if;
					
				when SM_INITALIZE  =>
					if Controller_SPIACTION = '1' then
						SPIMACHINE_STATE <= SM_WRITE;
						Controller_Wait <= '0';
                    else
						SPIMACHINE_STATE <= SM_INITALIZE;
                    end if;
               PauseCounter_Output <= unsigned(PauseCounter);

				when SM_WRITE  =>
					if (Controller_Busy = '0') AND (SPI_START = '0') AND (SPI_START_LAST = '0') then
						SPI_START <= '1';
						SPIMACHINE_STATE <= SM_LDAC;
					else
						SPIMACHINE_STATE <= SM_WRITE;
					end if;
					
				when SM_LDAC => 
					if (Controller_Busy = '0') AND (SPI_START = '0') AND (SPI_START_LAST = '0') then
						NOT_LDAC <= '0';
                  SPIMACHINE_STATE <= SM_COOLDOWN;
					else
						SPIMACHINE_STATE <= SM_LDAC;
					end if;
               ResetCounter <= (others => '0');

                when SM_COOLDOWN => 
					if ResetCounter = COOLDOWN_COUNTER then
						SPIMACHINE_STATE <= SM_WAIT;
                  ResetCounter <= (others => '0');
					else
						SPIMACHINE_STATE <= SM_COOLDOWN;
					end if;

                when SM_WAIT => 
					if ResetCounter = PauseCounter_Output then
						SPIMACHINE_STATE <= SM_INITALIZE;
					else
						SPIMACHINE_STATE <= SM_WAIT;
					end if;
            
				--when others => 
					--SPIMACHINE_STATE <= SM_RESET;
			end case;
		end if;
	
	end process DAC_FSM;
end Behavioral;

