library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity DACSwitchCore is
    Port ( 
			-- Controll data from an external data source 
			INPUT_FULL   						    : out STD_LOGIC;
			INPUT_DATA         				        : in  STD_LOGIC_VECTOR(8 downto 0);
			INPUT_DATA_VALID    				    : in  STD_LOGIC;
			INPUT_DATA_CLK    				        : in  STD_LOGIC;

			-- SPI and Co
			NOT_LDAC		    					: out STD_LOGIC; 
			NOT_RESET_DAC	    				    : out STD_LOGIC; 
			MOSI									: out STD_LOGIC;
			CS										: out STD_LOGIC;
			SCLK									: out STD_LOGIC;

			Switch_CS								: out STD_LOGIC_VECTOR(15 downto 0);
			Switch_A								: out STD_LOGIC_VECTOR(15 downto 0);
			Switch_B								: out STD_LOGIC_VECTOR(15 downto 0);
			Switch_C								: out STD_LOGIC_VECTOR(15 downto 0);
			Switch_D								: out STD_LOGIC_VECTOR(15 downto 0);

            UpdateSwitch                            : out STD_LOGIC;
			
			Reset 								    : in  STD_LOGIC;
			Clk 									: in  STD_LOGIC
		);
end DACSwitchCore;

architecture Behavioral of DACSwitchCore is

            signal INPUT_FULL_INTERNAL   				: STD_LOGIC := '0';
			signal INPUT_DATA_TG         				: STD_LOGIC_VECTOR(8 downto 0) := (others => '0');
			signal INPUT_DATA_VALID_TG    			    : STD_LOGIC := '0';

			signal INPUT_DATA_INTERNAL      			: STD_LOGIC_VECTOR(8 downto 0) := (others => '0');
			signal INPUT_DATA_VALID_INTERNAL			: STD_LOGIC := '0';
			signal INPUT_DATA_CLK_INTERNAL			    : STD_LOGIC := '0';

			signal Controller_ChannelID_EXPORT 		    : STD_LOGIC_VECTOR(3 downto 0) := (others => '0');
			signal Controller_Value_EXPORT			    : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
			signal Controller_Valid_EXPORT			    : STD_LOGIC := '0';
			signal Controller_SetMode_EXPORT			: STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
			signal Controller_Wait						: STD_LOGIC := '0';
			signal Controller_SPIACTION				    : STD_LOGIC := '0';
			signal PauseCounter							: STD_LOGIC_VECTOR(39 downto 0) := (others => '0');	

            constant TestMode 							: STD_LOGIC := '0';		
begin
	-- Export the internal variables 
	INPUT_FULL <= INPUT_FULL_INTERNAL;

	DataSelectionProcess : process(INPUT_DATA, INPUT_DATA_VALID, 
											INPUT_DATA_TG, INPUT_DATA_VALID_TG,
											INPUT_DATA_INTERNAL, INPUT_DATA_VALID_INTERNAL, 
											INPUT_DATA_CLK_INTERNAL, Clk, INPUT_DATA_CLK)
	begin
		if TestMode = '1' then
			INPUT_DATA_INTERNAL <= INPUT_DATA_TG;
			INPUT_DATA_VALID_INTERNAL <= INPUT_DATA_VALID_TG;
			INPUT_DATA_CLK_INTERNAL <= Clk;
		else
			INPUT_DATA_INTERNAL <= INPUT_DATA;	
			INPUT_DATA_VALID_INTERNAL <= INPUT_DATA_VALID;
			INPUT_DATA_CLK_INTERNAL <= INPUT_DATA_CLK;

		end if;
	end process DataSelectionProcess;

	TestGen : entity work.TestPackageGenerator
    Port MAP ( 
	 
            INPUT_FULL        				    => INPUT_FULL_INTERNAL,
			INPUT_DATA         				    => INPUT_DATA_TG,
			INPUT_DATA_VALID    				=> INPUT_DATA_VALID_TG,

			CLK									=> CLK,
			Reset 								=> Reset
		);
		
	MyPackageDis : entity work.PackageDisassembler
    Port MAP ( 
            INPUT_FULL        				    => INPUT_FULL_INTERNAL,
			INPUT_CLK							=> INPUT_DATA_CLK_INTERNAL,
			INPUT_DATA         				    => INPUT_DATA_INTERNAL,
			INPUT_DATA_VALID    				=> INPUT_DATA_VALID_INTERNAL,

			Controller_ChannelID_EXPORT 	    => Controller_ChannelID_EXPORT,
			Controller_Value_EXPORT			    => Controller_Value_EXPORT,
			Controller_Valid_EXPORT			    => Controller_Valid_EXPORT,
			Controller_SetMode_EXPORT		    => Controller_SetMode_EXPORT,
			Controller_Wait					    => Controller_Wait,
			Controller_SPIACTION				=> Controller_SPIACTION,
			
			PauseCounter						=> PauseCounter,
			
            Switch_CS							=> Switch_CS,
			Switch_A							=> Switch_A,
			Switch_B							=> Switch_B,
			Switch_C							=> Switch_C,
			Switch_D							=> Switch_D,
			
			Reset 			     				=> Reset,
			Clk 					  			=> Clk
		);		


	MyAD5360 : entity work.AD5360Programmer
    Port MAP ( 

				-- The Data for programming the DAC 
				Controller_ChannelID 			=> Controller_ChannelID_EXPORT,
				Controller_Value				=> Controller_Value_EXPORT,
				Controller_Valid				=> Controller_Valid_EXPORT,
				Controller_SetMode				=> Controller_SetMode_EXPORT,
				PauseCounter					=> PauseCounter,
				
				Controller_Wait					=> Controller_Wait,
				Controller_SPIACTION			=> Controller_SPIACTION,
				
				-- SPI and Co
				NOT_LDAC		    			=> NOT_LDAC, 
				NOT_RESET_DAC	    			=> NOT_RESET_DAC,
				MOSI							=> MOSI,
				CS								=> CS,
				SCLK							=> SCLK,

                UpdateSwitch                    => UpdateSwitch,
			
				Clk 							=> Clk,
                Reset 			    			=> Reset
	 );

end Behavioral;

