library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;


entity DACTestProgrammer is
    Port ( 
		NOT_LDAC 	: out STD_LOGIC;
		NOT_RESET 	: out STD_LOGIC;
		NOT_CLEAR 	: out STD_LOGIC;
		NOT_OD 		: out STD_LOGIC;

		-- Output
		MOSI			: out STD_LOGIC;
		SCLK			: out STD_LOGIC;
		CS				: out STD_LOGIC;
		
		-- Usual ports 
		NOTReset		: in  STD_LOGIC;		
		Clk 			: in  STD_LOGIC
	);
end DACTestProgrammer;

architecture Behavioral of DACTestProgrammer is

-- Register: "0100"
	constant Command_INITIALIZE : std_logic_vector(15 downto 0) := "00000" &
		"00" & "0" & "0" & "1" & "0" & "00" & "010";

	constant NOT_LDAC_VALUE 		: std_logic := '0';
	--constant NOT_RESET_VALUE 		: std_logic := '1';
	constant NOT_CLEAR_VALUE 		: std_logic := '1';
	constant NOT_OD_VALUE 			: std_logic := '1';


	signal CommandID 					: STD_LOGIC_VECTOR (2 downto 0) := (others => '0');
	signal Value 						: STD_LOGIC_VECTOR (15 downto 0) := (others => '0');
	
	signal Counter 					: unsigned (15 downto 0) := (others => '0');
	
		-- Signal to be ready
	signal NextValue					: STD_LOGIC := '0';

		-- Start SPI Transmission 
	signal SPI_START  				: STD_LOGIC := '0';
	signal SPI_START_LAST			: STD_LOGIC := '0';

	signal Reset 						: STD_LOGIC := '0';

	type STATE_Type is (
		SM_RESET,
		SM_PAUSE,
		SM_INITALIZE,
		SM_VALUES
	);	
	signal SPIMACHINE_STATE : STATE_TYPE;	
	
	-- Wait until the reset of the FPGA
	signal ResetCounter 				: unsigned (25 downto 0) := (others => '0');
	constant ResetCounter_END		: unsigned (25 downto 0) := to_unsigned(12000000,26);
	-- Use for the simulation:
	-- constant ResetCounter_END		: unsigned (25 downto 0) := to_unsigned(12,26);
begin
	NOT_LDAC <= NOT_LDAC_VALUE;
	--NOT_RESET <= NOT_RESET_VALUE;
	NOT_CLEAR <= NOT_CLEAR_VALUE;
	NOT_OD <= NOT_OD_VALUE;
	Reset <= NOT NOTReset;

	FSM : process(Reset,Clk)
	begin
		if (Reset = '1') then
			SPIMACHINE_STATE <= SM_RESET;
			SPI_START_LAST <= '0';
			SPI_START <= '0';
			Counter <= (others => '0');
			CommandID <= (others => '0');
			Value <= (others => '0');
			ResetCounter <= (others => '0');
            NOT_RESET <= '0';
			
		elsif (rising_edge(Clk)) then
			-- Default 
			SPIMACHINE_STATE <= SM_RESET;
			SPI_START_LAST <= SPI_START;
			SPI_START <= '0';
			Counter <= Counter;
			CommandID <= (others => '0');
			Value <= (others => '0');
			ResetCounter <= ResetCounter + 1;
            NOT_RESET <= '1';

			case SPIMACHINE_STATE is
				when SM_RESET =>	
					SPIMACHINE_STATE <= SM_PAUSE;
					Counter <= (others => '0');
					ResetCounter <= (others => '0');
					
				when SM_PAUSE => 
					if ResetCounter = ResetCounter_END then
						SPIMACHINE_STATE <= SM_INITALIZE;
					else
						SPIMACHINE_STATE <= SM_PAUSE;
					end if;
					
				when SM_INITALIZE  =>
					if NextValue = '1' then
						SPIMACHINE_STATE <= SM_VALUES;
						SPI_START <= '1';
					else
						SPIMACHINE_STATE <= SM_INITALIZE;
					end if;
					Value <= Command_INITIALIZE;							
					CommandID <= "011";
				
				when SM_VALUES  =>
					SPIMACHINE_STATE <= SM_VALUES;
					Value <= std_logic_vector(Counter);
					CommandID <= "010";
					if (NextValue = '1') AND (SPI_START = '0') AND (SPI_START_LAST = '0') then
						SPI_START <= '1';
						Counter <= Counter + 1;
					else
						null;
					end if;
					
				when others => 
					SPIMACHINE_STATE <= SM_RESET;
			end case;
		end if;
	
	end process FSM;

MyDAC: entity work.DACSPI
    Port MAP ( 
		CommandID 	=> CommandID,
		Value 		=> Value,
		NextValue	=> NextValue,
		SPI_START  	=> SPI_START,
		MOSI			=> MOSI,
		SCLK			=> SCLK,
		CS				=> CS,
		Reset			=> Reset,
		Clk 			=> Clk
		);
	
end Behavioral;

