library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity DAVROT_BURST_HELPER is
		Generic (
				FIFO_WIDTH : integer := 8
			);
		Port ( 
			Write_Data				    : in  STD_LOGIC_VECTOR(FIFO_WIDTH-1 downto 0);
			Write_WriteEnable		    : in  STD_LOGIC;
			Write_Full				    : out STD_LOGIC;
				
			
			FIFO_DIN				    : out STD_LOGIC_VECTOR(FIFO_WIDTH-1 downto 0);
			FIFO_FULL				    : in  STD_LOGIC;
			FIFO_WR_EN				    : out STD_LOGIC;

            EXTRA_OUPUT_BUFFER_EMPTY    : out STD_LOGIC;
			
			Clk						    : in  STD_LOGIC;
			Reset					    : in  STD_LOGIC
		);
end DAVROT_BURST_HELPER;

architecture Behavioral of DAVROT_BURST_HELPER is
	signal OUPUT_BUFFER_0 		: STD_LOGIC_VECTOR(FIFO_WIDTH-1 downto 0) := (others => '0');
	signal OUPUT_BUFFER_0_VALID: STD_LOGIC := '0';
	signal OUPUT_BUFFER_1 		: STD_LOGIC_VECTOR(FIFO_WIDTH-1 downto 0) := (others => '0');
	signal OUPUT_BUFFER_1_VALID: STD_LOGIC := '0';
	signal OUPUT_BUFFER_2 		: STD_LOGIC_VECTOR(FIFO_WIDTH-1 downto 0) := (others => '0');
	signal OUPUT_BUFFER_2_VALID: STD_LOGIC := '0';
	signal OUPUT_BUFFER_3 		: STD_LOGIC_VECTOR(FIFO_WIDTH-1 downto 0) := (others => '0');
	signal OUPUT_BUFFER_3_VALID: STD_LOGIC := '0';
	
	signal EXTRA_OUPUT_BUFFER_EMPTY_INTERNAL : STD_LOGIC := '0';
	
	signal Write_Full_Internal : STD_LOGIC := '0';
	
begin

	Write_Full 		<= Write_Full_Internal OR FIFO_FULL;

	EXTRA_OUPUT_BUFFER_EMPTY_INTERNAL <= OUPUT_BUFFER_0_VALID OR OUPUT_BUFFER_1_VALID OR 
										 OUPUT_BUFFER_2_VALID OR OUPUT_BUFFER_3_VALID;
										 
    EXTRA_OUPUT_BUFFER_EMPTY <= EXTRA_OUPUT_BUFFER_EMPTY_INTERNAL;

   ALMOSTFULL : process (Reset, Clk)
   begin
       if Reset = '1' then
			OUPUT_BUFFER_0 		<= (others => '0');
			OUPUT_BUFFER_0_VALID <= '0';
			OUPUT_BUFFER_1 		<= (others => '0');
			OUPUT_BUFFER_1_VALID <= '0';
			OUPUT_BUFFER_2 		<= (others => '0');
			OUPUT_BUFFER_2_VALID <= '0';
			OUPUT_BUFFER_3 		<= (others => '0');
			OUPUT_BUFFER_3_VALID <= '0';		 
			FIFO_DIN 				<= (others => '0');
			FIFO_WR_EN 				<= '0';
			
			Write_Full_Internal <= '1';
			
	    elsif rising_edge(Clk) then
		 
				-- If the FIFO is full, then we don't want any extra data as soon as possible 
				if FIFO_FULL = '1' then
					Write_Full_Internal <= '1';
				-- In the case where the FIFO wasn't full, we keep the status
				elsif Write_Full_Internal = '0' then
					Write_Full_Internal <= '0';
				-- At this position the FIFO is not full
				-- In the case the extra buffer is empty now 
				-- then we allow new data 
				elsif EXTRA_OUPUT_BUFFER_EMPTY_INTERNAL = '0' then
					Write_Full_Internal <= '0';
				-- Nothing happend that could change our mind 
				else
					Write_Full_Internal <= Write_Full_Internal;
				end if;
		 		 
			-- DATA PIPE LINE 
			if FIFO_FULL = '0' then
				
				OUPUT_BUFFER_0_VALID <= Write_WriteEnable;
				if Write_WriteEnable = '1' then
					OUPUT_BUFFER_0 		<= Write_Data;
				else
					OUPUT_BUFFER_0 		<= (others => '0');
				end if;
				OUPUT_BUFFER_1 		    <= OUPUT_BUFFER_0;
				OUPUT_BUFFER_1_VALID    <= OUPUT_BUFFER_0_VALID;
				OUPUT_BUFFER_2 		    <= OUPUT_BUFFER_1;
				OUPUT_BUFFER_2_VALID    <= OUPUT_BUFFER_1_VALID;
				OUPUT_BUFFER_3 		    <= OUPUT_BUFFER_2;
				OUPUT_BUFFER_3_VALID    <= OUPUT_BUFFER_2_VALID;
				FIFO_DIN 			    <= OUPUT_BUFFER_3;
				FIFO_WR_EN 				<= OUPUT_BUFFER_3_VALID;			
			else
				-- When full, then hold the pipeline
				OUPUT_BUFFER_0 		<= OUPUT_BUFFER_0;
				OUPUT_BUFFER_0_VALID <= OUPUT_BUFFER_0_VALID;
				OUPUT_BUFFER_1 		<= OUPUT_BUFFER_1;
				OUPUT_BUFFER_1_VALID <= OUPUT_BUFFER_1_VALID;
				OUPUT_BUFFER_2 		<= OUPUT_BUFFER_2;
				OUPUT_BUFFER_2_VALID <= OUPUT_BUFFER_2_VALID;
				OUPUT_BUFFER_3 		<= OUPUT_BUFFER_3;
				OUPUT_BUFFER_3_VALID <= OUPUT_BUFFER_3_VALID;
				FIFO_DIN 				<= (others => '0');
				FIFO_WR_EN 				<= '0';
			end if;
		 
       end if;
   end process ALMOSTFULL;
	
end Behavioral;

