library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
 
entity FIFO_INTERFACE_RD is
		Generic (
				FIFO_WIDTH : integer := 8
			);
		Port ( 
			Read_Data				: out STD_LOGIC_VECTOR(FIFO_WIDTH-1 downto 0);
			Read_ReadEnable		    : in  STD_LOGIC;
			Read_Valid				: out STD_LOGIC;
			Read_Clk				: in  STD_LOGIC;

			FIFO_RD_EN				: out STD_LOGIC;
			FIFO_DOUT				: in  STD_LOGIC_VECTOR(FIFO_WIDTH-1 downto 0);
			FIFO_EMPTY				: in  STD_LOGIC;
			
			Reset 					: in  STD_LOGIC
		);
end FIFO_INTERFACE_RD;

architecture Behavioral of FIFO_INTERFACE_RD is
	
	signal Read_Valid_Internal 	    : STD_LOGIC := '0';
	signal Read_Data_Internal		: STD_LOGIC_VECTOR(FIFO_WIDTH-1 downto 0) := (others => '0');
	
	signal FIFO_RD_EN_Internal 	    : STD_LOGIC := '0';
	signal FIFO_EMPTY_Internal 	    : STD_LOGIC := '0';
	
	signal FIFO_VALID 				: STD_LOGIC := '0';
    signal MIDDLE_VALID  			: STD_LOGIC := '0';
	signal UPDATE_MIDDLE			: STD_LOGIC := '0';
	signal UPDATE_DOUT				: STD_LOGIC := '0';
	signal CHECK_A					: STD_LOGIC := '0';
    signal MIDDLE_DATA 				: STD_LOGIC_VECTOR(FIFO_WIDTH-1 downto 0) := (others => '0');
	signal FIFO_DOUT_Internal 		: STD_LOGIC_VECTOR(FIFO_WIDTH-1 downto 0) := (others => '0');	
	
begin

	Read_Valid					<= Read_Valid_Internal;
	Read_Data 					<= Read_Data_Internal;

	FIFO_RD_EN 					<= FIFO_RD_EN_Internal;
	
	FIFO_DOUT_Internal 		    <= FIFO_DOUT;
	FIFO_EMPTY_Internal 		<= FIFO_EMPTY;

    
	-----------------------------------------------------------------
	---- Reading (based on http://www.billauer.co.il/reg_fifo.html )
	-----------------------------------------------------------------
	---- Fall Through Logic 
	UPDATE_MIDDLE <= FIFO_VALID AND CHECK_A;
	CHECK_A <= '1' when (MIDDLE_VALID = UPDATE_DOUT) else '0';
	UPDATE_DOUT <= (MIDDLE_VALID OR FIFO_VALID) AND (Read_ReadEnable OR (NOT Read_Valid_Internal));
	FIFO_RD_EN_Internal <= (NOT FIFO_EMPTY_Internal) AND NOT(MIDDLE_VALID AND Read_Valid_Internal AND FIFO_VALID);
		
   HandlingTheRead : process (Reset,Read_Clk)
   begin
       if Reset = '1' then
            FIFO_VALID 				<= '0';
            MIDDLE_VALID 			<= '0';
            Read_Valid_Internal	<= '0';
            Read_Data_Internal	<= (others => '0');
            MIDDLE_DATA 			<= (others => '0');
		 elsif rising_edge(Read_Clk) then

			if (UPDATE_MIDDLE = '1') then
				MIDDLE_DATA <= FIFO_DOUT_Internal;
			else
				MIDDLE_DATA <= MIDDLE_DATA;
			end if;
            
         if (UPDATE_DOUT = '1') AND (MIDDLE_VALID = '1') then
				Read_Data_Internal <=  MIDDLE_DATA ;
			elsif (UPDATE_DOUT = '1') AND (MIDDLE_VALID = '0') then
				Read_Data_Internal <=  FIFO_DOUT_Internal;
			else
				Read_Data_Internal <= Read_Data_Internal;
			end if;
            
         if (FIFO_RD_EN_Internal = '1') then
				FIFO_VALID <= '1';
			elsif (UPDATE_MIDDLE = '1') OR (UPDATE_DOUT = '1') then
				FIFO_VALID <= '0';
			else
				FIFO_VALID <= FIFO_VALID;
			end if;
            
         if (UPDATE_MIDDLE = '1') then
				MIDDLE_VALID <= '1';
         elsif (UPDATE_DOUT = '1') then
				MIDDLE_VALID <= '0';
			else
				MIDDLE_VALID <= MIDDLE_VALID;
			end if;
            
         if (UPDATE_DOUT = '1') then
				Read_Valid_Internal <= '1';
         elsif (Read_ReadEnable = '1') then
				Read_Valid_Internal <= '0';
			else 
				Read_Valid_Internal <= Read_Valid_Internal;
			end if;
		end if;
	end process HandlingTheRead;
	
end Behavioral;