library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity HV2201 is
port (
        DATA_0                          : in  STD_LOGIC_VECTOR(7 downto 0);
        DATA_1                          : in  STD_LOGIC_VECTOR(7 downto 0);
        DATA_2                          : in  STD_LOGIC_VECTOR(7 downto 0);
        DATA_3                          : in  STD_LOGIC_VECTOR(7 downto 0);
        DATA_4                          : in  STD_LOGIC_VECTOR(7 downto 0);
        DATA_5                          : in  STD_LOGIC_VECTOR(7 downto 0);
        DATA_6                          : in  STD_LOGIC_VECTOR(7 downto 0);
        DATA_7                          : in  STD_LOGIC_VECTOR(7 downto 0);

        START                           : in  STD_LOGIC;
        BUSY                            : out STD_LOGIC;

        -- Switches
        SW_CLR                          : out STD_LOGIC_VECTOR(1 downto 0);
        SW_NOT_LE                       : out STD_LOGIC_VECTOR(1 downto 0);
        SW_CLK                          : out STD_LOGIC_VECTOR(1 downto 0);
        SW_DIN                          : out STD_LOGIC_VECTOR(7 downto 0);

		Clk                             : in  STD_LOGIC;
		Reset                           : in STD_LOGIC
);
end HV2201;
architecture BEHAVIOURALS of HV2201 is
    signal DATA_0_COPY       : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
    signal DATA_1_COPY       : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
    signal DATA_2_COPY       : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
    signal DATA_3_COPY       : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
    signal DATA_4_COPY       : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
    signal DATA_5_COPY       : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
    signal DATA_6_COPY       : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
    signal DATA_7_COPY       : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');

    signal ClkHalf           : STD_LOGIC := '0';   

	type STATE_Type is (
		SM_RESET,
        SM_IDLE,
        SM_BYTE_0_A,
        SM_BYTE_0_B,
        SM_BYTE_1_A,
        SM_BYTE_1_B,
        SM_BYTE_2_A,
        SM_BYTE_2_B,
        SM_BYTE_3_A,
        SM_BYTE_3_B,
        SM_BYTE_4_A,
        SM_BYTE_4_B,
        SM_BYTE_5_A,
        SM_BYTE_5_B,
        SM_BYTE_6_A,
        SM_BYTE_6_B,
        SM_BYTE_7_A,
        SM_BYTE_7_B,
        SM_EXPORT_A,
        SM_EXPORT_B,
        SM_EXPORT_C,
        SM_EXPORT_D,
        SM_COOLDOWN
	);	
	signal SPIMACHINE_STATE : STATE_TYPE;

begin

        HalfClock : process(Reset,Clk)
        begin
            if (Reset = '1') then 
                ClkHalf <= '0';
            elsif rising_edge(Clk) then
                ClkHalf <= NOT ClkHalf;
            end if;
        end process HalfClock;

        MyShiftFSM : process(Reset,ClkHalf)
        begin
            if (Reset = '1') then 
                SPIMACHINE_STATE <= SM_RESET;


                SW_CLR <= (others => '0');
                SW_NOT_LE <= (others => '1');
                SW_CLK <= (others => '0');
                SW_DIN <= (others => '0');

                BUSY <= '1';

                DATA_0_COPY <= (others => '0');
                DATA_1_COPY <= (others => '0');
                DATA_2_COPY <= (others => '0');
                DATA_3_COPY <= (others => '0');
                DATA_4_COPY <= (others => '0');
                DATA_5_COPY <= (others => '0');
                DATA_6_COPY <= (others => '0');
                DATA_7_COPY <= (others => '0');

            elsif rising_edge(ClkHalf) then
                -- Default:
                SPIMACHINE_STATE <= SM_RESET;

                BUSY <= '1';

                SW_CLR <= (others => '0');
                SW_NOT_LE <= (others => '1');
                SW_CLK <= (others => '0');
                SW_DIN <= (others => '0');

                DATA_0_COPY <= DATA_0_COPY;
                DATA_1_COPY <= DATA_1_COPY;
                DATA_2_COPY <= DATA_2_COPY;                
                DATA_3_COPY <= DATA_3_COPY;
                DATA_4_COPY <= DATA_4_COPY;                
                DATA_5_COPY <= DATA_5_COPY;
                DATA_6_COPY <= DATA_6_COPY;                
                DATA_7_COPY <= DATA_7_COPY;

                case SPIMACHINE_STATE is 

                    when SM_RESET =>
                        SPIMACHINE_STATE <= SM_IDLE;

                    when SM_IDLE =>
                        
                        if START = '1' then
                            SPIMACHINE_STATE <= SM_BYTE_0_A;
                        else
                            SPIMACHINE_STATE <= SM_IDLE;
                        end if;
                        BUSY <= '0';
                        DATA_0_COPY <= DATA_0;
                        DATA_1_COPY <= DATA_1;
                        DATA_2_COPY <= DATA_2;                
                        DATA_3_COPY <= DATA_3;
                        DATA_4_COPY <= DATA_4;                
                        DATA_5_COPY <= DATA_5;
                        DATA_6_COPY <= DATA_6;                
                        DATA_7_COPY <= DATA_7;

                    when SM_BYTE_0_A =>
                        SPIMACHINE_STATE <= SM_BYTE_0_B;
                        SW_DIN <= DATA_7_COPY(7) & DATA_6_COPY(7) & DATA_5_COPY(7) & DATA_4_COPY(7) &
                                  DATA_3_COPY(7) & DATA_2_COPY(7) & DATA_1_COPY(7) & DATA_0_COPY(7);

                    when SM_BYTE_0_B =>
                        SPIMACHINE_STATE <= SM_BYTE_1_A;
                        SW_DIN <= DATA_7_COPY(7) & DATA_6_COPY(7) & DATA_5_COPY(7) & DATA_4_COPY(7) &
                                  DATA_3_COPY(7) & DATA_2_COPY(7) & DATA_1_COPY(7) & DATA_0_COPY(7);
                        SW_CLK <= (others => '1');

                    when SM_BYTE_1_A =>
                        SPIMACHINE_STATE <= SM_BYTE_1_B;
                        SW_DIN <= DATA_7_COPY(6) & DATA_6_COPY(6) & DATA_5_COPY(6) & DATA_4_COPY(6) &
                                  DATA_3_COPY(6) & DATA_2_COPY(6) & DATA_1_COPY(6) & DATA_0_COPY(6);

                    when SM_BYTE_1_B =>
                        SPIMACHINE_STATE <= SM_BYTE_2_A;
                        SW_DIN <= DATA_7_COPY(6) & DATA_6_COPY(6) & DATA_5_COPY(6) & DATA_4_COPY(6) &
                                  DATA_3_COPY(6) & DATA_2_COPY(6) & DATA_1_COPY(6) & DATA_0_COPY(6);
                        SW_CLK <= (others => '1');

                    when SM_BYTE_2_A =>
                        SPIMACHINE_STATE <= SM_BYTE_2_B;
                        SW_DIN <= DATA_7_COPY(5) & DATA_6_COPY(5) & DATA_5_COPY(5) & DATA_4_COPY(5) &
                                  DATA_3_COPY(5) & DATA_2_COPY(5) & DATA_1_COPY(5) & DATA_0_COPY(5);

                    when SM_BYTE_2_B =>
                        SPIMACHINE_STATE <= SM_BYTE_3_A;
                        SW_DIN <= DATA_7_COPY(5) & DATA_6_COPY(5) & DATA_5_COPY(5) & DATA_4_COPY(5) &
                                  DATA_3_COPY(5) & DATA_2_COPY(5) & DATA_1_COPY(5) & DATA_0_COPY(5);
                        SW_CLK <= (others => '1');

                    when SM_BYTE_3_A =>
                        SPIMACHINE_STATE <= SM_BYTE_3_B;
                        SW_DIN <= DATA_7_COPY(4) & DATA_6_COPY(4) & DATA_5_COPY(4) & DATA_4_COPY(4) &
                                  DATA_3_COPY(4) & DATA_2_COPY(4) & DATA_1_COPY(4) & DATA_0_COPY(4);

                    when SM_BYTE_3_B =>
                        SPIMACHINE_STATE <= SM_BYTE_4_A;
                        SW_DIN <= DATA_7_COPY(4) & DATA_6_COPY(4) & DATA_5_COPY(4) & DATA_4_COPY(4) &
                                  DATA_3_COPY(4) & DATA_2_COPY(4) & DATA_1_COPY(4) & DATA_0_COPY(4);
                        SW_CLK <= (others => '1');

                    when SM_BYTE_4_A =>
                        SPIMACHINE_STATE <= SM_BYTE_4_B;
                        SW_DIN <= DATA_7_COPY(3) & DATA_6_COPY(3) & DATA_5_COPY(3) & DATA_4_COPY(3) &
                                  DATA_3_COPY(3) & DATA_2_COPY(3) & DATA_1_COPY(3) & DATA_0_COPY(3);

                    when SM_BYTE_4_B =>
                        SPIMACHINE_STATE <= SM_BYTE_5_A;
                        SW_DIN <= DATA_7_COPY(3) & DATA_6_COPY(3) & DATA_5_COPY(3) & DATA_4_COPY(3) &
                                  DATA_3_COPY(3) & DATA_2_COPY(3) & DATA_1_COPY(3) & DATA_0_COPY(3);
                        SW_CLK <= (others => '1');

                    when SM_BYTE_5_A =>
                        SPIMACHINE_STATE <= SM_BYTE_5_B;
                        SW_DIN <= DATA_7_COPY(2) & DATA_6_COPY(2) & DATA_5_COPY(2) & DATA_4_COPY(2) &
                                  DATA_3_COPY(2) & DATA_2_COPY(2) & DATA_1_COPY(2) & DATA_0_COPY(2);

                    when SM_BYTE_5_B =>
                        SPIMACHINE_STATE <= SM_BYTE_6_A;
                        SW_DIN <= DATA_7_COPY(2) & DATA_6_COPY(2) & DATA_5_COPY(2) & DATA_4_COPY(2) &
                                  DATA_3_COPY(2) & DATA_2_COPY(2) & DATA_1_COPY(2) & DATA_0_COPY(2);
                        SW_CLK <= (others => '1');

                    when SM_BYTE_6_A =>
                        SPIMACHINE_STATE <= SM_BYTE_6_B;
                        SW_DIN <= DATA_7_COPY(1) & DATA_6_COPY(1) & DATA_5_COPY(1) & DATA_4_COPY(1) &
                                  DATA_3_COPY(1) & DATA_2_COPY(1) & DATA_1_COPY(1) & DATA_0_COPY(1);

                    when SM_BYTE_6_B =>
                        SPIMACHINE_STATE <= SM_BYTE_7_A;
                        SW_DIN <= DATA_7_COPY(1) & DATA_6_COPY(1) & DATA_5_COPY(1) & DATA_4_COPY(1) &
                                  DATA_3_COPY(1) & DATA_2_COPY(1) & DATA_1_COPY(1) & DATA_0_COPY(1);
                        SW_CLK <= (others => '1');

                    when SM_BYTE_7_A =>
                        SPIMACHINE_STATE <= SM_BYTE_7_B;
                        SW_DIN <= DATA_7_COPY(0) & DATA_6_COPY(0) & DATA_5_COPY(0) & DATA_4_COPY(0) &
                                  DATA_3_COPY(0) & DATA_2_COPY(0) & DATA_1_COPY(0) & DATA_0_COPY(0);

                    when SM_BYTE_7_B =>
                        SPIMACHINE_STATE <= SM_EXPORT_A;
                        SW_DIN <= DATA_7_COPY(0) & DATA_6_COPY(0) & DATA_5_COPY(0) & DATA_4_COPY(0) &
                                  DATA_3_COPY(0) & DATA_2_COPY(0) & DATA_1_COPY(0) & DATA_0_COPY(0);
                        SW_CLK <= (others => '1');

                    when SM_EXPORT_A =>
                        SPIMACHINE_STATE <= SM_EXPORT_B;

                    when SM_EXPORT_B =>
                        SPIMACHINE_STATE <= SM_EXPORT_C;

                    when SM_EXPORT_C =>
                        SPIMACHINE_STATE <= SM_EXPORT_D;
                        SW_NOT_LE <= (others => '0');

                    when SM_EXPORT_D =>
                        SPIMACHINE_STATE <= SM_COOLDOWN;
                        SW_NOT_LE <= (others => '0');

                    when SM_COOLDOWN =>
                        SPIMACHINE_STATE <= SM_IDLE;

                end case;

            end if;

        end process MyShiftFSM;
   
end ;
