library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity InputParser is
    Port ( 
		
		DATA_IN_EXT 						: in  STD_LOGIC_VECTOR(7 downto 0);
		DATA_IN_VALID_EXT 					: in  STD_LOGIC;		
		DATA_IN_READENABLE			    : out STD_LOGIC;
		
		-- For the strategy it is important that the PROG_FULL of the FIFO gives us some space :-) 
		DATA_OUT_EXT 						: out STD_LOGIC_VECTOR(8 downto 0);
		DATA_OUT_VALID_EXT 				    : out STD_LOGIC;
		DATA_OUT_FULL					: in  STD_LOGIC;
		
		ENTRAINMENT_FAILED			    : out STD_LOGIC;
		PACKET_BROKEN					: out STD_LOGIC;
		
		--  0000000 = Wildcard 
		MYADDRESS						: in STD_LOGIC_VECTOR(6 downto 0);

        DEBUG_DATA		                : out STD_LOGIC_VECTOR(8 downto 0);

		Clk                             : in  STD_LOGIC;
		Reset                           : in STD_LOGIC
		);
end InputParser;

architecture Behavioral of InputParser is

	type STATE_Type is (
		SM_RESET,
		SM_ENTRAINMENT_0,
		SM_ENTRAINMENT_1,
		SM_ENTRAINMENT_2,
		SM_ENTRAINMENT_3,
		SM_ENTRAINMENT_4,
		SM_ENTRAINMENT_5,
		SM_ENTRAINMENT_6,
		SM_ENTRAINMENT_7,
		SM_ENTRAINMENT_8,
		SM_ENTRAINMENT_9,
		SM_ENTRAINMENT_10,
		SM_ENTRAINMENT_11,
		SM_INIT_TRANSFER_A,
        SM_INIT_TRANSFER_B,
		SM_ADDRESS,
		SM_BYTE_01,
		SM_RESTOFTHEBLOCK
	);	
	signal SPIMACHINE_STATE : STATE_TYPE;
	
	constant ENTRAINMENT_0 		: STD_LOGIC_VECTOR(7 downto 0) := "10101010";
	constant ENTRAINMENT_1 		: STD_LOGIC_VECTOR(7 downto 0) := "11001010";
	constant ENTRAINMENT_2 		: STD_LOGIC_VECTOR(7 downto 0) := "11010101";
	constant ENTRAINMENT_3 		: STD_LOGIC_VECTOR(7 downto 0) := "01001010";
	constant ENTRAINMENT_4 		: STD_LOGIC_VECTOR(7 downto 0) := "10101001";
	constant ENTRAINMENT_5 		: STD_LOGIC_VECTOR(7 downto 0) := "10011100";
	constant ENTRAINMENT_6 		: STD_LOGIC_VECTOR(7 downto 0) := "11100011";
	constant ENTRAINMENT_7 		: STD_LOGIC_VECTOR(7 downto 0) := "10101111";
	constant ENTRAINMENT_8 		: STD_LOGIC_VECTOR(7 downto 0) := "00101010";
	constant ENTRAINMENT_9 		: STD_LOGIC_VECTOR(7 downto 0) := "00000000";
    constant ENTRAINMENT_10 	: STD_LOGIC_VECTOR(7 downto 0) := "00000000";
    constant ENTRAINMENT_11     : STD_LOGIC_VECTOR(7 downto 0) := "00000000";
	constant INIT_TRANSFER_A	: STD_LOGIC_VECTOR(7 downto 0) := "11111111";
    constant INIT_TRANSFER_B	: STD_LOGIC_VECTOR(7 downto 0) := "10101010";
	
	-- For the length of the package -1 or so... 
	constant COUNTER_MAX			: unsigned(7 downto 0) := to_unsigned(52,8);
	signal COUNTER : unsigned(7 downto 0) := (others => '0');
	
	signal DONT_CARE_BLOCK		: STD_LOGIC := '0';
	signal ENTRAINMENT_FAILED_INTERNAL : STD_LOGIC := '0';
	signal PACKET_BROKEN_INTERNAL : STD_LOGIC := '0';
	signal DATA_IN_READENABLE_INTERNAL : STD_LOGIC := '0';
    signal DEBUG_DATA_FSM : STD_LOGIC_VECTOR(8 downto 0) := (others => '0'); 
	signal DATA_IN 	: STD_LOGIC_VECTOR(7 downto 0) := (others => '0'); 
	signal DATA_IN_VALID  : STD_LOGIC := '0';		
	signal DATA_OUT : STD_LOGIC_VECTOR(8 downto 0) := (others => '0'); 
	signal DATA_OUT_VALID : STD_LOGIC := '1';

begin
	
	ENTRAINMENT_FAILED <= ENTRAINMENT_FAILED_INTERNAL;
	PACKET_BROKEN <= PACKET_BROKEN_INTERNAL;
	DATA_IN_READENABLE <= DATA_IN_READENABLE_INTERNAL;

	Parser: process(Reset,Clk)
	begin
		if (Reset = '1') then
			SPIMACHINE_STATE <= SM_RESET;
			ENTRAINMENT_FAILED_INTERNAL <= '0';
			PACKET_BROKEN_INTERNAL <= '0';
			DATA_OUT_VALID <= '0';
			DATA_OUT(8) <= '0';
			DONT_CARE_BLOCK <= '0';
			COUNTER <= (others => '0');
			DATA_OUT(7 downto 0)  <= (others => '0');
            DATA_IN_READENABLE_INTERNAL <= '0';
            DEBUG_DATA <= (others => '0');
            DEBUG_DATA_FSM <= (others => '0');

            DATA_IN <= (others => '0');
            DATA_IN_VALID <= '0';
            DATA_OUT <= (others => '0');
            DATA_OUT_VALID <= '0';
			
		elsif (rising_edge(Clk)) then
			SPIMACHINE_STATE <= SM_RESET;
			ENTRAINMENT_FAILED_INTERNAL <= ENTRAINMENT_FAILED_INTERNAL;
			PACKET_BROKEN_INTERNAL <= PACKET_BROKEN_INTERNAL;
			DATA_OUT_VALID <= '0';
			DATA_OUT(8) <= '0';
			DONT_CARE_BLOCK <= DONT_CARE_BLOCK;
			COUNTER <= COUNTER;
			DATA_OUT(7 downto 0) <= (others => '0');
			DATA_IN_READENABLE_INTERNAL <= '1'; --NOT DATA_OUT_FULL;
            
            DATA_IN <= DATA_IN_EXT;
            DATA_IN_VALID <= DATA_IN_VALID_EXT;

            DATA_OUT_EXT <= DATA_OUT;
            DATA_OUT_VALID_EXT <= DATA_OUT_VALID;


            DEBUG_DATA <= (others => '0');
            DEBUG_DATA_FSM <= (others => '0');
            -- DEBUG_DATA <= DATA_IN_VALID & DATA_IN;
            DEBUG_DATA <= DEBUG_DATA_FSM;
			case SPIMACHINE_STATE is
				when SM_RESET =>
					SPIMACHINE_STATE <= SM_ENTRAINMENT_0;
					
				when SM_ENTRAINMENT_0 =>
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') AND (DATA_IN = ENTRAINMENT_0) then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_1;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(1,8));
					elsif (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_0;
						ENTRAINMENT_FAILED_INTERNAL <= '1';
                        DEBUG_DATA_FSM <= "1" & DATA_IN; -- std_logic_vector(to_unsigned(0,8));
					else
						SPIMACHINE_STATE <= SM_ENTRAINMENT_0;
					end if;
					
				when SM_ENTRAINMENT_1 =>
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') AND (DATA_IN = ENTRAINMENT_1) then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_2;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(2,8));
					elsif (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_0;
						ENTRAINMENT_FAILED_INTERNAL <= '1';
                        DEBUG_DATA_FSM <= "1" & DATA_IN; -- std_logic_vector(to_unsigned(0,8));
					else
						SPIMACHINE_STATE <= SM_ENTRAINMENT_1;
					end if;
					
				when SM_ENTRAINMENT_2 =>
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') AND (DATA_IN = ENTRAINMENT_2) then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_3;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(3,8));
					elsif (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_0;
						ENTRAINMENT_FAILED_INTERNAL <= '1';
                        DEBUG_DATA_FSM <= "1" & DATA_IN; -- std_logic_vector(to_unsigned(0,8));
					else
						SPIMACHINE_STATE <= SM_ENTRAINMENT_2;
					end if;
					
				when SM_ENTRAINMENT_3 =>
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') AND (DATA_IN = ENTRAINMENT_3) then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_4;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(4,8));
					elsif (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_0;
						ENTRAINMENT_FAILED_INTERNAL <= '1';
                        DEBUG_DATA_FSM <= "1" & DATA_IN; -- std_logic_vector(to_unsigned(0,8));
					else
						SPIMACHINE_STATE <= SM_ENTRAINMENT_3;
					end if;				
				
				when SM_ENTRAINMENT_4 =>
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') AND (DATA_IN = ENTRAINMENT_4) then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_5;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(5,8));
					elsif (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_0;
						ENTRAINMENT_FAILED_INTERNAL <= '1';
                        DEBUG_DATA_FSM <= "1" & DATA_IN; -- std_logic_vector(to_unsigned(0,8));
					else
						SPIMACHINE_STATE <= SM_ENTRAINMENT_4;
					end if;	
					
				when SM_ENTRAINMENT_5 =>
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') AND (DATA_IN = ENTRAINMENT_5) then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_6;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(6,8));
					elsif (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_0;
						ENTRAINMENT_FAILED_INTERNAL <= '1';
                        DEBUG_DATA_FSM <= "1" & DATA_IN; -- std_logic_vector(to_unsigned(0,8));
					else
						SPIMACHINE_STATE <= SM_ENTRAINMENT_5;
					end if;	
					
				when SM_ENTRAINMENT_6 =>
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') AND (DATA_IN = ENTRAINMENT_6) then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_7;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(7,8));
					elsif (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_0;
						ENTRAINMENT_FAILED_INTERNAL <= '1';
                        DEBUG_DATA_FSM <= "1" & DATA_IN; -- & std_logic_vector(to_unsigned(0,8));
					else
						SPIMACHINE_STATE <= SM_ENTRAINMENT_6;
					end if;	
					
				when SM_ENTRAINMENT_7 =>
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') AND (DATA_IN = ENTRAINMENT_7) then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_8;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(8,8));
					elsif (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_0;
						ENTRAINMENT_FAILED_INTERNAL <= '1';
                        DEBUG_DATA_FSM <= "1" & DATA_IN; -- std_logic_vector(to_unsigned(0,8));
					else
						SPIMACHINE_STATE <= SM_ENTRAINMENT_7;
					end if;	
					
				when SM_ENTRAINMENT_8 =>
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') AND (DATA_IN = ENTRAINMENT_8) then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_9;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(9,8));
					elsif (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_0;
						ENTRAINMENT_FAILED_INTERNAL <= '1';
                        DEBUG_DATA_FSM <= "1" & DATA_IN; -- std_logic_vector(to_unsigned(0,8));
					else
						SPIMACHINE_STATE <= SM_ENTRAINMENT_8;
					end if;
					
				when SM_ENTRAINMENT_9 =>				
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') AND (DATA_IN = ENTRAINMENT_9) then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_10;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(10,8));
						PACKET_BROKEN_INTERNAL <= '0';
					elsif (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_0;
						ENTRAINMENT_FAILED_INTERNAL <= '1';
                        DEBUG_DATA_FSM <= "1" & DATA_IN; -- std_logic_vector(to_unsigned(0,8));
					else
						SPIMACHINE_STATE <= SM_ENTRAINMENT_9;
					end if;
				
				when SM_ENTRAINMENT_10 =>				
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') AND (DATA_IN = ENTRAINMENT_10) then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_11;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(11,8));
						PACKET_BROKEN_INTERNAL <= '0';
					elsif (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_0;
						ENTRAINMENT_FAILED_INTERNAL <= '1';
                        DEBUG_DATA_FSM <= "1" & DATA_IN; -- std_logic_vector(to_unsigned(0,8));
					else
						SPIMACHINE_STATE <= SM_ENTRAINMENT_10;
					end if;

				when SM_ENTRAINMENT_11 =>				
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') AND (DATA_IN = ENTRAINMENT_11) then
						SPIMACHINE_STATE <= SM_INIT_TRANSFER_A;
						PACKET_BROKEN_INTERNAL <= '0';
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(20,8));
					elsif (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_0;
						ENTRAINMENT_FAILED_INTERNAL <= '1';
                        DEBUG_DATA_FSM <= "1" & DATA_IN; -- std_logic_vector(to_unsigned(0,8));
					else
						SPIMACHINE_STATE <= SM_ENTRAINMENT_11;
					end if;

				when SM_INIT_TRANSFER_A =>  
					ENTRAINMENT_FAILED_INTERNAL <= '0';
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') AND (DATA_IN = INIT_TRANSFER_A) then
						SPIMACHINE_STATE <= SM_INIT_TRANSFER_B;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(21,8));
					elsif (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_0;
						PACKET_BROKEN_INTERNAL <= '1';
                        DEBUG_DATA_FSM <= "1" & DATA_IN; -- std_logic_vector(to_unsigned(0,8));
					else
						SPIMACHINE_STATE <= SM_INIT_TRANSFER_A;
					end if;

				when SM_INIT_TRANSFER_B =>  
					ENTRAINMENT_FAILED_INTERNAL <= '0';
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') AND (DATA_IN = INIT_TRANSFER_B) then
						SPIMACHINE_STATE <= SM_ADDRESS;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(22,8));
					elsif (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_ENTRAINMENT_0;
						PACKET_BROKEN_INTERNAL <= '1';
                        DEBUG_DATA_FSM <= "1" & DATA_IN; -- std_logic_vector(to_unsigned(0,8));
					else
						SPIMACHINE_STATE <= SM_INIT_TRANSFER_B;
					end if;

					
				when SM_ADDRESS => 
					-- Here is the place to add more commands
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') AND((DATA_IN(6 downto 0) = MYADDRESS) OR (DATA_IN(6 downto 0) = "0000000"))
						AND (DATA_IN(7) = '0') then
						SPIMACHINE_STATE <= SM_BYTE_01;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(23,8));
						DONT_CARE_BLOCK <= '1';
					elsif (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_BYTE_01;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(24,8));
						DONT_CARE_BLOCK <= '0';
					else 
						SPIMACHINE_STATE <= SM_ADDRESS; 
					end if;
					
				when SM_BYTE_01 => 
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_RESTOFTHEBLOCK;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(25,8));
						DATA_OUT_VALID <= DONT_CARE_BLOCK;
						COUNTER <= to_unsigned(1,8);
					else 
						SPIMACHINE_STATE <= SM_BYTE_01; 
					end if;	
					DATA_OUT(8) <= '1';
					DATA_OUT(7 downto 0) <= DATA_IN;
					
				when SM_RESTOFTHEBLOCK => 
					if (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') AND (COUNTER = COUNTER_MAX) then
						SPIMACHINE_STATE <= SM_INIT_TRANSFER_A;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(26,8));
						DATA_OUT_VALID <= DONT_CARE_BLOCK;

					elsif (DATA_IN_VALID = '1') AND (DATA_IN_READENABLE_INTERNAL = '1') then
						SPIMACHINE_STATE <= SM_RESTOFTHEBLOCK;
                        DEBUG_DATA_FSM <= "1" & std_logic_vector(to_unsigned(27,8));
						DATA_OUT_VALID <= DONT_CARE_BLOCK;
						COUNTER <= COUNTER + 1;

					else 
						SPIMACHINE_STATE <= SM_RESTOFTHEBLOCK; 
					end if;	
					DATA_OUT(7 downto 0) <= DATA_IN;
					
			end case;
		end if;
	end process Parser;


end Behavioral;

