library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity PackageDisassembler is
    Port ( 
            INPUT_FULL        				    : out STD_LOGIC;
			INPUT_CLK							: in  STD_LOGIC;
			INPUT_DATA         				    : in  STD_LOGIC_VECTOR(8 downto 0);
			INPUT_DATA_VALID    				: in  STD_LOGIC;

			Controller_ChannelID_EXPORT 	    : out STD_LOGIC_VECTOR(3 downto 0);
			Controller_Value_EXPORT			    : out STD_LOGIC_VECTOR(15 downto 0);
			Controller_Valid_EXPORT			    : out STD_LOGIC;
			Controller_SetMode_EXPORT		    : out STD_LOGIC_VECTOR(1 downto 0);
			Controller_Wait					    : in  STD_LOGIC;
			Controller_SPIACTION				: out STD_LOGIC;
			
			PauseCounter						: out STD_LOGIC_VECTOR(39 downto 0);
			
            Switch_CS							: out STD_LOGIC_VECTOR(15 downto 0);
			Switch_A							: out STD_LOGIC_VECTOR(15 downto 0);
			Switch_B							: out STD_LOGIC_VECTOR(15 downto 0);
			Switch_C							: out STD_LOGIC_VECTOR(15 downto 0);
			Switch_D							: out STD_LOGIC_VECTOR(15 downto 0);

			Reset 			     				: in  STD_LOGIC;
			Clk 					  			: in  STD_LOGIC
		);
end PackageDisassembler;

architecture Behavioral of PackageDisassembler is

	signal Controller_ChannelID 				: unsigned(3 downto 0) := (others => '0');
	signal Controller_Value						: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal Controller_Valid						: STD_LOGIC := '0';
	signal Controller_SetMode					: STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	signal INPUT_READEN_INTERNAL				: STD_LOGIC := '0';
	signal INPUT_READEN_LAST					: STD_LOGIC := '0';
	signal PauseCounter_INTERNAL				: STD_LOGIC_VECTOR(39 downto 0) := (others => '0');
	signal Controller_SPIACTION_INTERNAL	: STD_LOGIC := '0';
		

	signal Mode_00_Input : STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	signal Mode_01_Input : STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	signal Mode_02_Input : STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	signal Mode_03_Input : STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	signal Mode_04_Input : STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	signal Mode_05_Input : STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	signal Mode_06_Input : STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	signal Mode_07_Input : STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	signal Mode_08_Input : STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	signal Mode_09_Input : STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	signal Mode_10_Input : STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	signal Mode_11_Input : STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	signal Mode_12_Input : STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	signal Mode_13_Input : STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	signal Mode_14_Input : STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	signal Mode_15_Input : STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
	
	signal READENABLE_SAFE : STD_LOGIC := '0';
    
    signal Switch_CS_INTERNAL : STD_LOGIC_VECTOR(15 downto 0)   := (others => '0');
	signal Switch_A_INTERNAL : STD_LOGIC_VECTOR(15 downto 0)    := (others => '0');
	signal Switch_B_INTERNAL : STD_LOGIC_VECTOR(15 downto 0)	:= (others => '0');
	signal Switch_C_INTERNAL : STD_LOGIC_VECTOR(15 downto 0)	:= (others => '0');
	signal Switch_D_INTERNAL : STD_LOGIC_VECTOR(15 downto 0)	:= (others => '0');
	
	type STATE_Type is (
			SM_RESET,
			SM_WAIT_FOR_NEWFRAME,
			SM_BYTE_01,
			SM_BYTE_02,
			SM_BYTE_03,
			SM_BYTE_04,
			SM_BYTE_05,
			SM_BYTE_06,
			SM_BYTE_07,
			SM_BYTE_08,
			SM_BYTE_09,
			SM_BYTE_10,
			SM_BYTE_11,
			SM_BYTE_12,
			SM_BYTE_13,
			SM_BYTE_14,
			SM_BYTE_15,
			SM_BYTE_16,
			SM_BYTE_17,
			SM_BYTE_18,
			SM_BYTE_19,
			SM_BYTE_20,
			SM_BYTE_21,
			SM_BYTE_22,
			SM_BYTE_23,
			SM_BYTE_24,
			SM_BYTE_25,
			SM_BYTE_26,
			SM_BYTE_27,
			SM_BYTE_28,
			SM_BYTE_29,
			SM_BYTE_30,
			SM_BYTE_31,
			SM_BYTE_32,
			SM_BYTE_33,
			SM_BYTE_34,
			SM_BYTE_35,
			SM_BYTE_36,
			SM_BYTE_37,
			SM_BYTE_38,
			SM_BYTE_39,
			SM_BYTE_40,
			SM_BYTE_41,
			SM_BYTE_42,
			SM_BYTE_43,
			SM_BYTE_44,
			SM_BYTE_45,
			SM_BYTE_46,
			SM_BYTE_47,
			SM_BYTE_48,
			SM_BYTE_49,
            SM_BYTE_50,
            SM_BYTE_51,
            SM_BYTE_52,
            SM_BYTE_53,
			SM_TRANSMIT,
			SM_COOLDOWN			
	);	
	signal SPIMACHINE_STATE : STATE_TYPE;		
	
	-- FIFO:
	signal dout 								: STD_LOGIC_VECTOR(8 downto 0) := (others => '0');
	signal INPUT_VALID         				    : STD_LOGIC := '0';
	signal INPUT_VALUE 							: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
	signal INPUT_NEW_FRAME 						: STD_LOGIC := '0';
	signal FIFO_EMPTY                           : STD_LOGIC := '0';
    signal FIFO_RD_EN                           : STD_LOGIC := '0';
begin
	-- Export to external ports
	Controller_ChannelID_EXPORT <= STD_LOGIC_VECTOR(Controller_ChannelID);
	Controller_Value_EXPORT <= Controller_Value;
	Controller_Valid_EXPORT <= Controller_Valid;
	Controller_SetMode_EXPORT <= Controller_SetMode;

	PauseCounter <= PauseCounter_INTERNAL;
	Controller_SPIACTION <= Controller_SPIACTION_INTERNAL;
    Switch_CS <= Switch_CS_INTERNAL;
	Switch_A <= Switch_A_INTERNAL;
	Switch_B <= Switch_B_INTERNAL;
	Switch_C <= Switch_C_INTERNAL;
	Switch_D <= Switch_D_INTERNAL;

    -- XILINX
	--MyFIFO : entity work.FIFO_DISASSEMBLER
	--PORT MAP (
		--rst => Reset,
		--wr_clk => INPUT_CLK,
		--rd_clk => Clk,
		--din => INPUT_DATA,
		--wr_en => INPUT_DATA_VALID,
		--rd_en => INPUT_READEN_INTERNAL,
		--dout => dout,
		--full => open,
		--empty => FIFO_EMPTY,
		--valid => open,
		--prog_full => INPUT_FULL
	--);

    MyFIFO : entity work.FIFO_DISASSEMBLE
    port MAP( 
          DATA   => INPUT_DATA,
          Q      => dout,
          WE     => INPUT_DATA_VALID,
          RE     => FIFO_RD_EN,
          WCLOCK => INPUT_CLK,
          RCLOCK => Clk,
          FULL   => open,
          EMPTY  => FIFO_EMPTY,
          RESET  => Reset,
          AEMPTY => open,
          AFULL => INPUT_FULL
        );


    

	INPUT_VALUE <= dout(7 downto 0);
	INPUT_NEW_FRAME <= dout(8);
	
	-- Handling the delay of the FIFO
	READENABLE_SAFE <= INPUT_READEN_INTERNAL OR INPUT_READEN_LAST;


    -- Synchronous FIFO for SmartFusion, IGLOO, ProASIC3 and Fusion Summary
    -- PIPLINE: NO
    -- Reset: Active High
    -- Write Enable: Active High
    -- Read Enable: Active High 
    FIFO_RD_EN <= (NOT FIFO_EMPTY) AND ((NOT INPUT_VALID ) OR INPUT_READEN_INTERNAL);

    FIFOFALLTHROUGH : process (Reset,Clk)
    begin
        if Reset = '1' then

            INPUT_VALID <= '0';

        elsif rising_edge(Clk) then

            INPUT_VALID <= INPUT_VALID;
            
            if fifo_rd_en = '1' then
                INPUT_VALID <= '1';
            elsif INPUT_READEN_INTERNAL = '1' then
                INPUT_VALID <= '0';
            else
                null;
            end if;

        end if;

    end process FIFOFALLTHROUGH;

	
    InputHandler : process(Reset,Clk)
    begin
        if (Reset = '1') then

            SPIMACHINE_STATE <= SM_RESET;
				Controller_ChannelID <= (others => '0');
				Controller_Value <= (others => '0');
				Controller_Valid <= '0';
				Controller_SetMode <= (others => '0');
				INPUT_READEN_INTERNAL <= '0';
				INPUT_READEN_LAST <= '0';
				PauseCounter_INTERNAL <= (others => '0');
				Controller_SPIACTION_INTERNAL	<= '0';				
				Mode_00_Input <= (others => '0');
				Mode_01_Input <= (others => '0');
				Mode_02_Input <= (others => '0');
				Mode_03_Input <= (others => '0');
				Mode_04_Input <= (others => '0');
				Mode_05_Input <= (others => '0');
				Mode_06_Input <= (others => '0');
				Mode_07_Input <= (others => '0');
				Mode_08_Input <= (others => '0');
				Mode_09_Input <= (others => '0');
				Mode_10_Input <= (others => '0');
				Mode_11_Input <= (others => '0');
				Mode_12_Input <= (others => '0');
				Mode_13_Input <= (others => '0');
				Mode_14_Input <= (others => '0');
				Mode_15_Input <= (others => '0');
                Switch_CS_INTERNAL <= (others => '0');
				Switch_A_INTERNAL <= (others => '0');
				Switch_B_INTERNAL <= (others => '0');
				Switch_C_INTERNAL <= (others => '0');
				Switch_D_INTERNAL <= (others => '0');				

        elsif (rising_edge(Clk)) then
            -- DEFAULT:
            SPIMACHINE_STATE <= SM_RESET;
				Controller_ChannelID <= Controller_ChannelID;
				Controller_Value <= Controller_Value;
				Controller_Valid <= '0';
				Controller_SetMode <= Controller_SetMode;
				INPUT_READEN_INTERNAL <= '0';
				INPUT_READEN_LAST <= INPUT_READEN_INTERNAL;
				PauseCounter_INTERNAL <= PauseCounter_INTERNAL;
				Controller_SPIACTION_INTERNAL	<= Controller_SPIACTION_INTERNAL;	
				Mode_00_Input <= Mode_00_Input;
				Mode_01_Input <= Mode_01_Input;
				Mode_02_Input <= Mode_02_Input;
				Mode_03_Input <= Mode_03_Input;
				Mode_04_Input <= Mode_04_Input;
				Mode_05_Input <= Mode_05_Input;
				Mode_06_Input <= Mode_06_Input;
				Mode_07_Input <= Mode_07_Input;
				Mode_08_Input <= Mode_08_Input;
				Mode_09_Input <= Mode_09_Input;
				Mode_10_Input <= Mode_10_Input;
				Mode_11_Input <= Mode_11_Input;
				Mode_12_Input <= Mode_12_Input;
				Mode_13_Input <= Mode_13_Input;
				Mode_14_Input <= Mode_14_Input;
				Mode_15_Input <= Mode_15_Input;
                Switch_CS_INTERNAL <= Switch_CS_INTERNAL;
				Switch_A_INTERNAL <= Switch_A_INTERNAL;
				Switch_B_INTERNAL <= Switch_B_INTERNAL;
				Switch_C_INTERNAL <= Switch_C_INTERNAL;
				Switch_D_INTERNAL <= Switch_D_INTERNAL;						
				
            case SPIMACHINE_STATE is

						when SM_RESET => 
							SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;

						when SM_WAIT_FOR_NEWFRAME => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_01;
							else
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							end if;
							
						when SM_BYTE_01 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_02;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_01;
							end if;
							Mode_00_Input <= INPUT_VALUE(1 downto 0);
							Mode_01_Input <= INPUT_VALUE(3 downto 2);
							Mode_02_Input <= INPUT_VALUE(5 downto 4);
							Mode_03_Input <= INPUT_VALUE(7 downto 6);
							
						when SM_BYTE_02 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_03;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_02;
							end if;
							Mode_04_Input <= INPUT_VALUE(1 downto 0);
							Mode_05_Input <= INPUT_VALUE(3 downto 2);
							Mode_06_Input <= INPUT_VALUE(5 downto 4);
							Mode_07_Input <= INPUT_VALUE(7 downto 6);
							
						when SM_BYTE_03 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_04;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_03;
							end if;
							Mode_08_Input <= INPUT_VALUE(1 downto 0);
							Mode_09_Input <= INPUT_VALUE(3 downto 2);
							Mode_10_Input <= INPUT_VALUE(5 downto 4);
							Mode_11_Input <= INPUT_VALUE(7 downto 6);
						
						when SM_BYTE_04 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_05;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_04;
							end if;
							Mode_12_Input <= INPUT_VALUE(1 downto 0);
							Mode_13_Input <= INPUT_VALUE(3 downto 2);
							Mode_14_Input <= INPUT_VALUE(5 downto 4);
							Mode_15_Input <= INPUT_VALUE(7 downto 6);
							
						when SM_BYTE_05 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_06;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_05;
							end if;
							PauseCounter_INTERNAL(7 downto 0) <= INPUT_VALUE;
							
						when SM_BYTE_06 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_07;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_06;
							end if;
							PauseCounter_INTERNAL(15 downto 8) <= INPUT_VALUE;
							
						when SM_BYTE_07 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_08;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_07;
							end if;
							PauseCounter_INTERNAL(23 downto 16) <= INPUT_VALUE;
						
						when SM_BYTE_08 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_09;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_08;
							end if;
							PauseCounter_INTERNAL(31 downto 24) <= INPUT_VALUE;
						
						when SM_BYTE_09 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_10;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_09;
							end if;
							PauseCounter_INTERNAL(39 downto 32) <= INPUT_VALUE;
							
						-- 00
						when SM_BYTE_10 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_11;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_10;
							end if;
							Controller_Value(7 downto 0) <= INPUT_VALUE;
							
						when SM_BYTE_11 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_12;
								INPUT_READEN_INTERNAL <= '1';
								Controller_Valid <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_11;
							end if;
							Controller_ChannelID <= to_unsigned(0,4);
							Controller_Value <= INPUT_VALUE & Controller_Value(7 downto 0);
							Controller_SetMode <= Mode_00_Input;
							
						-- 01	
						when SM_BYTE_12 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_13;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_12;
							end if;
							Controller_Value(7 downto 0) <= INPUT_VALUE;
						
						when SM_BYTE_13 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_14;
								INPUT_READEN_INTERNAL <= '1';
								Controller_Valid <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_13;
							end if;
							Controller_ChannelID <= to_unsigned(1,4);
							Controller_Value <= INPUT_VALUE & Controller_Value(7 downto 0);
							Controller_SetMode <= Mode_01_Input;
							
						-- 02	
						when SM_BYTE_14 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_15;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_14;
							end if;
							Controller_Value(7 downto 0) <= INPUT_VALUE;
						
						when SM_BYTE_15 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_16;
								INPUT_READEN_INTERNAL <= '1';
								Controller_Valid <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_15;
							end if;
							Controller_ChannelID <= to_unsigned(2,4);
							Controller_Value <= INPUT_VALUE & Controller_Value(7 downto 0);
							Controller_SetMode <= Mode_02_Input;		
							
						-- 03
						when SM_BYTE_16 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_17;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_16;
							end if;
							Controller_Value(7 downto 0) <= INPUT_VALUE;
						
						when SM_BYTE_17 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_18;
								INPUT_READEN_INTERNAL <= '1';
								Controller_Valid <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_17;
							end if;
							Controller_ChannelID <= to_unsigned(3,4);
							Controller_Value <= INPUT_VALUE & Controller_Value(7 downto 0);
							Controller_SetMode <= Mode_03_Input;		
							
						-- 04
						when SM_BYTE_18 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_19;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_18;
							end if;
							Controller_Value(7 downto 0) <= INPUT_VALUE;
						
						when SM_BYTE_19 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_20;
								INPUT_READEN_INTERNAL <= '1';
								Controller_Valid <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_19;
							end if;
							Controller_ChannelID <= to_unsigned(4,4);
							Controller_Value <= INPUT_VALUE & Controller_Value(7 downto 0);
							Controller_SetMode <= Mode_04_Input;	
							
						-- 05
						when SM_BYTE_20 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_21;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_20;
							end if;
							Controller_Value(7 downto 0) <= INPUT_VALUE;
						
						when SM_BYTE_21 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_22;
								INPUT_READEN_INTERNAL <= '1';
								Controller_Valid <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_21;
							end if;
							Controller_ChannelID <= to_unsigned(5,4);
							Controller_Value <= INPUT_VALUE & Controller_Value(7 downto 0);
							Controller_SetMode <= Mode_05_Input;				
							
						-- 06
						when SM_BYTE_22 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_23;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_22;
							end if;
							Controller_Value(7 downto 0) <= INPUT_VALUE;
						
						when SM_BYTE_23 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_24;
								INPUT_READEN_INTERNAL <= '1';
								Controller_Valid <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_23;
							end if;
							Controller_ChannelID <= to_unsigned(6,4);
							Controller_Value <= INPUT_VALUE & Controller_Value(7 downto 0);
							Controller_SetMode <= Mode_06_Input;
							
						-- 07
						when SM_BYTE_24 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_25;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_24;
							end if;						
							Controller_Value(7 downto 0) <= INPUT_VALUE;
						
						when SM_BYTE_25 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_26;
								INPUT_READEN_INTERNAL <= '1';
								Controller_Valid <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_25;
							end if;
							Controller_ChannelID <= to_unsigned(7,4);
							Controller_Value <= INPUT_VALUE & Controller_Value(7 downto 0);
							Controller_SetMode <= Mode_07_Input;	
							
						-- 08
						when SM_BYTE_26 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_27;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_26;
							end if;
							Controller_Value(7 downto 0) <= INPUT_VALUE;
							
						when SM_BYTE_27 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_28;
								INPUT_READEN_INTERNAL <= '1';
								Controller_Valid <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_27;
							end if;
							Controller_ChannelID <= to_unsigned(8,4);
							Controller_Value <= INPUT_VALUE & Controller_Value(7 downto 0);
							Controller_SetMode <= Mode_08_Input;			
							
						-- 09	
						when SM_BYTE_28 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_29;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_28;
							end if;
							Controller_Value(7 downto 0) <= INPUT_VALUE;
							
						when SM_BYTE_29 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_30;
								INPUT_READEN_INTERNAL <= '1';
								Controller_Valid <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_29;
							end if;
							Controller_ChannelID <= to_unsigned(9,4);
							Controller_Value <= INPUT_VALUE & Controller_Value(7 downto 0);
							Controller_SetMode <= Mode_09_Input;			
							
						-- 10
						when SM_BYTE_30 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_31;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_30;
							end if;
							Controller_Value(7 downto 0) <= INPUT_VALUE;
							
						when SM_BYTE_31 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_32;
								INPUT_READEN_INTERNAL <= '1';
								Controller_Valid <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_31;
							end if;
							Controller_ChannelID <= to_unsigned(10,4);
							Controller_Value <= INPUT_VALUE & Controller_Value(7 downto 0);
							Controller_SetMode <= Mode_10_Input;	
							
						-- 11
						when SM_BYTE_32 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_33;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_32;
							end if;
							Controller_Value(7 downto 0) <= INPUT_VALUE;
							
						when SM_BYTE_33 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_34;
								INPUT_READEN_INTERNAL <= '1';
								Controller_Valid <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_33;
							end if;
							Controller_ChannelID <= to_unsigned(11,4);
							Controller_Value <= INPUT_VALUE & Controller_Value(7 downto 0);
							Controller_SetMode <= Mode_11_Input;				
							
						-- 12
						when SM_BYTE_34 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_35;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_34;
							end if;
							Controller_Value(7 downto 0) <= INPUT_VALUE;
							
						when SM_BYTE_35 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_36;
								INPUT_READEN_INTERNAL <= '1';
								Controller_Valid <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_35;
							end if;
							Controller_ChannelID <= to_unsigned(12,4);
							Controller_Value <= INPUT_VALUE & Controller_Value(7 downto 0);
							Controller_SetMode <= Mode_12_Input;	
							
						-- 13
						when SM_BYTE_36 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_37;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_36;
							end if;
							Controller_Value(7 downto 0) <= INPUT_VALUE;
							
						when SM_BYTE_37 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_38;
								INPUT_READEN_INTERNAL <= '1';
								Controller_Valid <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_37;
							end if;
							Controller_ChannelID <= to_unsigned(13,4);
							Controller_Value <= INPUT_VALUE & Controller_Value(7 downto 0);
							Controller_SetMode <= Mode_13_Input;			
							
						-- 14
						when SM_BYTE_38 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_39;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_38;
							end if;
							Controller_Value(7 downto 0) <= INPUT_VALUE;
							
						when SM_BYTE_39 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_40;
								INPUT_READEN_INTERNAL <= '1';
								Controller_Valid <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_39;
							end if;
							Controller_ChannelID <= to_unsigned(14,4);
							Controller_Value <= INPUT_VALUE & Controller_Value(7 downto 0);
							Controller_SetMode <= Mode_14_Input;	
							
						-- 15
						when SM_BYTE_40 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_41;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_40;
							end if;
							Controller_Value(7 downto 0) <= INPUT_VALUE;
							
						when SM_BYTE_41 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_42;
								INPUT_READEN_INTERNAL <= '1';
								Controller_Valid <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_41;
							end if;
							Controller_ChannelID <= to_unsigned(15,4);
							Controller_Value <= INPUT_VALUE & Controller_Value(7 downto 0);
							Controller_SetMode <= Mode_15_Input;			
							
--------------------------------------------------------------------------------------	

						-- SWITCH CS
						when SM_BYTE_42 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_43;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_42;
							end if;
							Switch_CS_INTERNAL(7 downto 0) <= INPUT_VALUE;
							
						when SM_BYTE_43 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_44;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_43;
							end if;
							Switch_CS_INTERNAL(15 downto 8) <= INPUT_VALUE;
							
						-- SWITCH A
						when SM_BYTE_44 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_45;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_44;
							end if;
							Switch_A_INTERNAL(7 downto 0) <= INPUT_VALUE;
							
						when SM_BYTE_45 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_46;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_45;
							end if;
							Switch_A_INTERNAL(15 downto 8) <= INPUT_VALUE;
						
						-- SWITCH B
						when SM_BYTE_46 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_47;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_46;
							end if;
							Switch_B_INTERNAL(7 downto 0) <= INPUT_VALUE;
						
						
						when SM_BYTE_47 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_48;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_47;
							end if;
							Switch_B_INTERNAL(15 downto 8) <= INPUT_VALUE;

						-- SWITCH C								
						when SM_BYTE_48 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_49;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_48;
							end if;
							Switch_C_INTERNAL(7 downto 0) <= INPUT_VALUE;
							
						when SM_BYTE_49 =>
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_50;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_49;
							end if;
							Switch_C_INTERNAL(15 downto 8) <= INPUT_VALUE;

						-- SWITCH D								
						when SM_BYTE_50 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_51;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_50;
							end if;
							Switch_D_INTERNAL(7 downto 0) <= INPUT_VALUE;
							
						when SM_BYTE_51 =>
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_52;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_51;
							end if;
							Switch_D_INTERNAL(15 downto 8) <= INPUT_VALUE;

						-- RESERVE								
						when SM_BYTE_52 => 
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_BYTE_53;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_52;
							end if;

							
						when SM_BYTE_53 =>
							if (INPUT_NEW_FRAME = '1') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
							elsif (INPUT_NEW_FRAME = '0') AND (INPUT_VALID = '1') AND (READENABLE_SAFE = '0') then 
								SPIMACHINE_STATE <= SM_TRANSMIT;
								INPUT_READEN_INTERNAL <= '1';
							else
								SPIMACHINE_STATE <= SM_BYTE_53;
							end if;
							
						when SM_TRANSMIT => 
							
                            Controller_SPIACTION_INTERNAL <= '1';
                            SPIMACHINE_STATE <= SM_COOLDOWN;
							
						when SM_COOLDOWN => 
							if Controller_Wait = '0' then
								SPIMACHINE_STATE <= SM_WAIT_FOR_NEWFRAME;
								Controller_SPIACTION_INTERNAL <= '0';
							else
								SPIMACHINE_STATE <= SM_COOLDOWN;
							end if;
							
                --when others => 
                    --SPIMACHINE_STATE <= SM_RESET;
             end case;

        end if;
        
    end process InputHandler;

end Behavioral;

