library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity ShiftReg74AHC595 is
port (
      
        DATA_A                          : in  STD_LOGIC_VECTOR(7 downto 0);
        DATA_B                          : in  STD_LOGIC_VECTOR(7 downto 0);

        START                           : in  STD_LOGIC;
        BUSY                            : out STD_LOGIC;

        CS_SRCLK                        : out STD_LOGIC;
        CS_RCLK                         : out STD_LOGIC;
        CS_SER                          : out STD_LOGIC_VECTOR(1 downto 0);

		Clk                             : in  STD_LOGIC;
		Reset                           : in STD_LOGIC

);
end ShiftReg74AHC595;

architecture BEHAVIOURAL of ShiftReg74AHC595 is

        signal DATA_A_COPY                          : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        signal DATA_B_COPY                          : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');


	type STATE_Type is (
		SM_RESET,
        SM_IDLE,
        SM_BYTE_0_A,
        SM_BYTE_0_B,
        SM_BYTE_1_A,
        SM_BYTE_1_B,
        SM_BYTE_2_A,
        SM_BYTE_2_B,
        SM_BYTE_3_A,
        SM_BYTE_3_B,
        SM_BYTE_4_A,
        SM_BYTE_4_B,
        SM_BYTE_5_A,
        SM_BYTE_5_B,
        SM_BYTE_6_A,
        SM_BYTE_6_B,
        SM_BYTE_7_A,
        SM_BYTE_7_B,
        SM_EXPORT_A,
        SM_EXPORT_B,
        SM_COOLDOWN
	);	
	signal SPIMACHINE_STATE : STATE_TYPE;

begin
        MyShiftFSM : process(Reset,Clk)
        begin
            if (Reset = '1') then 
                SPIMACHINE_STATE <= SM_RESET;

                CS_SRCLK <= '0';
                CS_RCLK <= '0';
                CS_SER <= "00";
                BUSY <= '1';

                DATA_A_COPY <= (others => '0');
                DATA_B_COPY <= (others => '0');

            elsif rising_edge(Clk) then
                -- Default:
                SPIMACHINE_STATE <= SM_RESET;

                CS_SRCLK <= '0';
                CS_RCLK <= '0';
                CS_SER <= "00";
                BUSY <= '1';

                DATA_A_COPY <= DATA_A_COPY;
                DATA_B_COPY <= DATA_B_COPY;

                case SPIMACHINE_STATE is 

                    when SM_RESET =>
                        SPIMACHINE_STATE <= SM_IDLE;

                    when SM_IDLE =>
                        
                        if START = '1' then
                            SPIMACHINE_STATE <= SM_BYTE_0_A;
                        else
                            SPIMACHINE_STATE <= SM_IDLE;
                        end if;
                        BUSY <= '0';
                        DATA_A_COPY <= DATA_A;
                        DATA_B_COPY <= DATA_B;

                    when SM_BYTE_0_A =>
                        SPIMACHINE_STATE <= SM_BYTE_0_B;
                        CS_SER <= DATA_B_COPY(7) & DATA_A_COPY(7);

                    when SM_BYTE_0_B =>
                        SPIMACHINE_STATE <= SM_BYTE_1_A;
                        CS_SER <= DATA_B_COPY(7) & DATA_A_COPY(7);
                        CS_SRCLK <= '1';

                    when SM_BYTE_1_A =>
                        SPIMACHINE_STATE <= SM_BYTE_1_B;
                        CS_SER <= DATA_B_COPY(6) & DATA_A_COPY(6);

                    when SM_BYTE_1_B =>
                        SPIMACHINE_STATE <= SM_BYTE_2_A;
                        CS_SER <= DATA_B_COPY(6) & DATA_A_COPY(6);
                        CS_SRCLK <= '1';

                    when SM_BYTE_2_A =>
                        SPIMACHINE_STATE <= SM_BYTE_2_B;
                        CS_SER <= DATA_B_COPY(5) & DATA_A_COPY(5);

                    when SM_BYTE_2_B =>
                        SPIMACHINE_STATE <= SM_BYTE_3_A;
                        CS_SER <= DATA_B_COPY(5) & DATA_A_COPY(5);
                        CS_SRCLK <= '1';

                    when SM_BYTE_3_A =>
                        SPIMACHINE_STATE <= SM_BYTE_3_B;
                        CS_SER <= DATA_B_COPY(4) & DATA_A_COPY(4);

                    when SM_BYTE_3_B =>
                        SPIMACHINE_STATE <= SM_BYTE_4_A;
                        CS_SER <= DATA_B_COPY(4) & DATA_A_COPY(4);
                        CS_SRCLK <= '1';

                    when SM_BYTE_4_A =>
                        SPIMACHINE_STATE <= SM_BYTE_4_B;
                        CS_SER <= DATA_B_COPY(3) & DATA_A_COPY(3);

                    when SM_BYTE_4_B =>
                        SPIMACHINE_STATE <= SM_BYTE_5_A;
                        CS_SER <= DATA_B_COPY(3) & DATA_A_COPY(3);
                        CS_SRCLK <= '1';

                    when SM_BYTE_5_A =>
                        SPIMACHINE_STATE <= SM_BYTE_5_B;
                        CS_SER <= DATA_B_COPY(2) & DATA_A_COPY(2);

                    when SM_BYTE_5_B =>
                        SPIMACHINE_STATE <= SM_BYTE_6_A;
                        CS_SER <= DATA_B_COPY(2) & DATA_A_COPY(2);
                        CS_SRCLK <= '1';

                    when SM_BYTE_6_A =>
                        SPIMACHINE_STATE <= SM_BYTE_6_B;
                        CS_SER <= DATA_B_COPY(1) & DATA_A_COPY(1);

                    when SM_BYTE_6_B =>
                        SPIMACHINE_STATE <= SM_BYTE_7_A;
                        CS_SER <= DATA_B_COPY(1) & DATA_A_COPY(1);
                        CS_SRCLK <= '1';

                    when SM_BYTE_7_A =>
                        SPIMACHINE_STATE <= SM_BYTE_7_B;
                        CS_SER <= DATA_B_COPY(0) & DATA_A_COPY(0);

                    when SM_BYTE_7_B =>
                        SPIMACHINE_STATE <= SM_EXPORT_A;
                        CS_SER <= DATA_B_COPY(0) & DATA_A_COPY(0);
                        CS_SRCLK <= '1';

                    when SM_EXPORT_A =>
                        SPIMACHINE_STATE <= SM_EXPORT_B;

                    when SM_EXPORT_B =>
                        SPIMACHINE_STATE <= SM_COOLDOWN;
                        CS_RCLK <= '1';

                    when SM_COOLDOWN =>
                        SPIMACHINE_STATE <= SM_IDLE;

                end case;

            end if;

        end process MyShiftFSM;


end;
