library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity SwitchControl is
port (

		Switch_CS								: in  STD_LOGIC_VECTOR(15 downto 0);
		Switch_A								: in  STD_LOGIC_VECTOR(15 downto 0);
		Switch_B								: in  STD_LOGIC_VECTOR(15 downto 0);
		Switch_C								: in  STD_LOGIC_VECTOR(15 downto 0);
		Switch_D								: in  STD_LOGIC_VECTOR(15 downto 0);

        UpdateSwitch                            : in  STD_LOGIC;

        -- Current source NOT_OD control
        CS_SRCLK                                : out STD_LOGIC;
        CS_RCLK                                 : out STD_LOGIC;
        CS_SER                                  : out STD_LOGIC_VECTOR(1 downto 0);
        CS_TFLAG                                : in  STD_LOGIC;


        -- Switches
        SW_CLR                                  : out STD_LOGIC_VECTOR(1 downto 0);
        SW_NOT_LE                               : out STD_LOGIC_VECTOR(1 downto 0);
        SW_CLK                                  : out STD_LOGIC_VECTOR(1 downto 0);
        SW_DIN                                  : out STD_LOGIC_VECTOR(7 downto 0);

        TEMP_PROBLEM                            : out STD_LOGIC;

		Clk                                     : in  STD_LOGIC;
		Reset                                   : in  STD_LOGIC
);
end SwitchControl;
architecture BEHAVIOURAL of SwitchControl is
    	signal Switch_CS_COPY				: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal Switch_A_COPY				: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal Switch_B_COPY				: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal Switch_C_COPY				: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal Switch_D_COPY				: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');

        signal Shift_DATA_A_PROG            : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        signal Shift_DATA_B_PROG            : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');

        signal DATA_0                       : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        signal DATA_1                       : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        signal DATA_2                       : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        signal DATA_3                       : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        signal DATA_4                       : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        signal DATA_5                       : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        signal DATA_6                       : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        signal DATA_7                       : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        
        signal DATA_0_PROG                  : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        signal DATA_1_PROG                  : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        signal DATA_2_PROG                  : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        signal DATA_3_PROG                  : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        signal DATA_4_PROG                  : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        signal DATA_5_PROG                  : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        signal DATA_6_PROG                  : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        signal DATA_7_PROG                  : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');

        signal SW_START                     : STD_LOGIC := '0';
        signal SW_BUSY                      : STD_LOGIC := '0';

        signal Shift_START                  : STD_LOGIC := '0';
        signal Signal_BUSY                  : STD_LOGIC := '0';
        signal TEMP_PROBLEM_FLAG            : STD_LOGIC := '0';

begin
    TEMP_PROBLEM <= TEMP_PROBLEM_FLAG;

    NewValue : process (Reset, Clk)
    begin
        if Reset = '1' then
            Switch_CS_COPY <= (others => '0');
            Switch_A_COPY <= (others => '0');
            Switch_B_COPY <= (others => '0');
            Switch_C_COPY <= (others => '0');
            Switch_D_COPY <= (others => '0');

        elsif rising_edge(Clk) then
            Switch_CS_COPY <= Switch_CS_COPY;
            Switch_A_COPY <= Switch_A_COPY;
            Switch_B_COPY <= Switch_B_COPY;
            Switch_C_COPY <= Switch_C_COPY;
            Switch_D_COPY <= Switch_D_COPY;

            if UpdateSwitch = '1' then
                Switch_CS_COPY <= Switch_CS;
                Switch_A_COPY <= Switch_A;
                Switch_B_COPY <= Switch_B;
                Switch_C_COPY <= Switch_C;
                Switch_D_COPY <= Switch_D;
            else
                null;
            end if;

        end if;
    end process NewValue;

    MyShiftReg74AHC595 : entity work.ShiftReg74AHC595
    port MAP (
      
        DATA_A                          => Shift_DATA_A_PROG,
        DATA_B                          => Shift_DATA_B_PROG,

        START                           => Shift_START,
        BUSY                            => Signal_BUSY,

        CS_SRCLK                        => CS_SRCLK,
        CS_RCLK                         => CS_RCLK,
        CS_SER                          => CS_SER,

		Clk                             => Clk,
		Reset                           => Reset

    );


    ShiftControl595 : process (Reset, Clk)
    begin
        if Reset = '1' then
            Shift_DATA_A_PROG <= (others => '0');
            Shift_DATA_B_PROG <= (others => '0');
            Shift_START <= '0';
            TEMP_PROBLEM_FLAG <= '0';

        elsif rising_edge(Clk) then
            -- Default:
            Shift_DATA_A_PROG <= Shift_DATA_A_PROG;
            Shift_DATA_B_PROG <= Shift_DATA_B_PROG;
            Shift_START <= '0';

            -- If there is a problem with the TEMPERATURE, 
            -- then we disable the module 
            TEMP_PROBLEM_FLAG <= '0';

            -- If there is a temperature problem then we 
            -- disable the current sources 
            if (CS_TFLAG = '0') then
                Shift_DATA_A_PROG <= (others => '0');
                Shift_DATA_B_PROG <= (others => '0');
                Shift_START <= '1';
                TEMP_PROBLEM_FLAG <= '1';
            elsif ((Switch_CS_COPY(7 downto 0) /= Shift_DATA_A_PROG) OR (Switch_CS_COPY(15 downto 8) /= Shift_DATA_B_PROG)) 
              AND (Signal_BUSY = '0') then
                Shift_DATA_A_PROG <= Switch_CS_COPY(7 downto 0);
                Shift_DATA_B_PROG <= Switch_CS_COPY(15 downto 8);
                Shift_START <= '1';
            else
                null;
            end if;
        end if;
    end process ShiftControl595;

MyHV2201 : entity work.HV2201
port MAP(
        DATA_0                          => DATA_0_PROG,
        DATA_1                          => DATA_1_PROG,
        DATA_2                          => DATA_2_PROG,
        DATA_3                          => DATA_3_PROG,
        DATA_4                          => DATA_4_PROG,
        DATA_5                          => DATA_5_PROG,
        DATA_6                          => DATA_6_PROG,
        DATA_7                          => DATA_7_PROG,

        START                           => SW_START,
        BUSY                            => SW_BUSY,

        -- Switches
        SW_CLR                          => SW_CLR,
        SW_NOT_LE                       => SW_NOT_LE,
        SW_CLK                          => SW_CLK,
        SW_DIN                          => SW_DIN,

		Clk                             => Clk,
		Reset                           => Reset
);

    -- 0
    DATA_0(7) <= Switch_A(0);
    DATA_0(0) <= Switch_A(1);
    DATA_0(6) <= Switch_B(0);
    DATA_0(1) <= Switch_B(1);
    DATA_0(5) <= Switch_C(0);
    DATA_0(2) <= Switch_C(1);
    DATA_0(4) <= Switch_D(0);
    DATA_0(3) <= Switch_D(1);

    -- 1
    DATA_1(7) <= Switch_A(2);
    DATA_1(0) <= Switch_A(3);
    DATA_1(6) <= Switch_B(2);
    DATA_1(1) <= Switch_B(3);
    DATA_1(5) <= Switch_C(2);
    DATA_1(2) <= Switch_C(3);
    DATA_1(4) <= Switch_D(2);
    DATA_1(3) <= Switch_D(3);

    -- 2
    DATA_2(7) <= Switch_A(4);
    DATA_2(0) <= Switch_A(5);
    DATA_2(6) <= Switch_B(4);
    DATA_2(1) <= Switch_B(5);
    DATA_2(5) <= Switch_C(4);
    DATA_2(2) <= Switch_C(5);
    DATA_2(4) <= Switch_D(4);
    DATA_2(3) <= Switch_D(5);

    -- 3
    DATA_3(7) <= Switch_A(6);
    DATA_3(0) <= Switch_A(7);
    DATA_3(6) <= Switch_B(6);
    DATA_3(1) <= Switch_B(7);
    DATA_3(5) <= Switch_C(6);
    DATA_3(2) <= Switch_C(7);
    DATA_3(4) <= Switch_D(6);
    DATA_3(3) <= Switch_D(7);

    -- 4
    DATA_4(7) <= Switch_A(8);
    DATA_4(0) <= Switch_A(9);
    DATA_4(6) <= Switch_B(8);
    DATA_4(1) <= Switch_B(9);
    DATA_4(5) <= Switch_C(8);
    DATA_4(2) <= Switch_C(9);
    DATA_4(4) <= Switch_D(8);
    DATA_4(3) <= Switch_D(9);

    -- 5
    DATA_5(7) <= Switch_A(10);
    DATA_5(0) <= Switch_A(11);
    DATA_5(6) <= Switch_B(10);
    DATA_5(1) <= Switch_B(11);
    DATA_5(5) <= Switch_C(10);
    DATA_5(2) <= Switch_C(11);
    DATA_5(4) <= Switch_D(10);
    DATA_5(3) <= Switch_D(11);

    -- 6
    DATA_6(7) <= Switch_A(12);
    DATA_6(0) <= Switch_A(13);
    DATA_6(6) <= Switch_B(12);
    DATA_6(1) <= Switch_B(13);
    DATA_6(5) <= Switch_C(12);
    DATA_6(2) <= Switch_C(13);
    DATA_6(4) <= Switch_D(12);
    DATA_6(3) <= Switch_D(13);

    -- 7
    DATA_7(7) <= Switch_A(14);
    DATA_7(0) <= Switch_A(15);
    DATA_7(6) <= Switch_B(14);
    DATA_7(1) <= Switch_B(15);
    DATA_7(5) <= Switch_C(14);
    DATA_7(2) <= Switch_C(15);
    DATA_7(4) <= Switch_D(14);
    DATA_7(3) <= Switch_D(15);

    ShiftControlHV2201 : process (Reset, Clk)
    begin
        if Reset = '1' then
            DATA_0_PROG <= (others => '0');
            DATA_1_PROG <= (others => '0');
            DATA_2_PROG <= (others => '0');
            DATA_3_PROG <= (others => '0');
            DATA_4_PROG <= (others => '0');
            DATA_5_PROG <= (others => '0');
            DATA_6_PROG <= (others => '0');
            DATA_7_PROG <= (others => '0');
            SW_START <= '0';

        elsif rising_edge(Clk) then
            -- Default:
            DATA_0_PROG <= DATA_0_PROG;
            DATA_1_PROG <= DATA_1_PROG;
            DATA_2_PROG <= DATA_2_PROG;
            DATA_3_PROG <= DATA_3_PROG;
            DATA_4_PROG <= DATA_4_PROG;
            DATA_5_PROG <= DATA_5_PROG;
            DATA_6_PROG <= DATA_6_PROG;
            DATA_7_PROG <= DATA_7_PROG;

            SW_START <= '0';

           
            if ((DATA_0_PROG /= DATA_0) OR
              (DATA_1_PROG /= DATA_1) OR
              (DATA_2_PROG /= DATA_2) OR
              (DATA_3_PROG /= DATA_3) OR
              (DATA_4_PROG /= DATA_4) OR
              (DATA_5_PROG /= DATA_5) OR
              (DATA_6_PROG /= DATA_6) OR
              (DATA_7_PROG /= DATA_7) ) 
              AND (SW_BUSY = '0') then
                DATA_0_PROG <= DATA_0;
                DATA_1_PROG <= DATA_1;
                DATA_2_PROG <= DATA_2;
                DATA_3_PROG <= DATA_3;
                DATA_4_PROG <= DATA_4;
                DATA_5_PROG <= DATA_5;
                DATA_6_PROG <= DATA_6;
                DATA_7_PROG <= DATA_7;
                SW_START <= '1';
            else
                null;
            end if;
        end if;
    end process ShiftControlHV2201;

end;
