library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;


entity FIFO_Interface_UUT is
end FIFO_Interface_UUT;

architecture behavioral of FIFO_Interface_UUT is

    signal Clk    : std_logic := '0';
    signal MasterCounter : unsigned(127 downto 0) := (others=>'0'); 
    signal Reset_Inverted : STD_LOGIC := '0';

    signal Counter : unsigned(7 downto 0) := (others=>'0'); 

	signal Read_Data_OUT		        : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
	signal Read_ReadEnable_OUT	        : STD_LOGIC := '0';
	signal Read_Valid_OUT		        : STD_LOGIC := '0';

	signal FIFO_DIN_OUT					: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
	signal FIFO_WR_EN_OUT				: STD_LOGIC := '0';
	signal FIFO_RD_EN_OUT				: STD_LOGIC := '0';
	signal FIFO_DOUT_OUT		        : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
	signal FIFO_FULL_OUT				: STD_LOGIC := '0';
	signal FIFO_EMPTY_OUT				: STD_LOGIC := '0';

    signal DATA_IN_DATA             : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
    signal DATA_IN_VALID            : STD_LOGIC := '0';
    signal DATA_IN_NEXT             : STD_LOGIC := '0';
    signal DATA_IN_STARTED          : STD_LOGIC := '0';

    signal DATA_OUT_DATA            : STD_LOGIC_VECTOR(7 downto 0)  := (others => '0');
    signal DATA_OUT_VALID           : STD_LOGIC := '0';
    signal DATA_OUT_NEXT            : STD_LOGIC := '0';
    signal DATA_OUT_STARTED         : STD_LOGIC := '0';

    signal USB_DATA            : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
    -- When high, do not read data
    signal USB_NOT_RXF         : STD_LOGIC := '1';
    -- When high, do not write data 
    signal USB_NOT_TXE         : STD_LOGIC := '1';
    -- When low, data is read 
    signal USB_NOT_RD          : STD_LOGIC := '1';
    -- When low, data is written 
    signal USB_NOT_WR          : STD_LOGIC := '1';

begin

 TheMasterClock : process
begin
	clockloop: for i in 0 to 25000 loop
		clk <= NOT clk;
		wait for 8ns;
	end loop clockloop;
	wait;
end process TheMasterClock; 


Stimulus : process(clk) 
begin
	if rising_edge(clk) then

		MasterCounter <= MasterCounter + 1;

        --FIFO_DIN_OUT <= STD_LOGIC_VECTOR(Counter);

        --if MasterCounter < to_unsigned(5,128) then
            --FIFO_WR_EN_OUT <= '0';
            --Counter <= (others => '0');
        --else 
            --if FIFO_FULL_OUT = '0' then
                --FIFO_WR_EN_OUT <= '1';
                --Counter <= Counter + 1;
            --else
                --FIFO_WR_EN_OUT <= '0';
                --Counter <= Counter;
            --end if;
--
        --end if;

 		--if MasterCounter < to_unsigned(50,128) then
			--Read_ReadEnable_OUT <= '0';
		--else
			--Read_ReadEnable_OUT <= '1';
		--end if;       

        USB_NOT_RXF <= '1';
        USB_NOT_TXE <= '1';

		if MasterCounter < to_unsigned(5,128) then
			Reset_Inverted <= '1';
		else
			Reset_Inverted <= '0';
		end if;


	end if;
end process Stimulus;

    FIFO_DIN_OUT <= DATA_OUT_DATA;
    FIFO_WR_EN_OUT <= DATA_OUT_VALID;
    DATA_OUT_NEXT <= NOT FIFO_FULL_OUT;

    DATA_IN_DATA <= Read_Data_OUT;
    DATA_IN_VALID <= Read_Valid_OUT;
    Read_ReadEnable_OUT <= DATA_IN_NEXT;

    MyFIFO_OUT : entity work.FIFO_OUT
    port MAP( 
          DATA   => FIFO_DIN_OUT,
          Q      => FIFO_DOUT_OUT,
          WE     => FIFO_WR_EN_OUT,
          RE     => FIFO_RD_EN_OUT,
          WCLOCK => Clk,
          RCLOCK => Clk,
          FULL   => open,
          EMPTY  => FIFO_EMPTY_OUT,
          RESET  => Reset_Inverted,
          AEMPTY => open,
          AFULL  => FIFO_FULL_OUT
        );

    FIFO_OUT_Controller : entity work.FIFO_INTERFACE_RD
		Port MAP( 

			Read_Data				=> Read_Data_OUT,
			Read_ReadEnable		    => Read_ReadEnable_OUT,
			Read_Valid				=> Read_Valid_OUT,
			Read_Clk				=> Clk,

			FIFO_RD_EN				=> FIFO_RD_EN_OUT,
			FIFO_DOUT				=> FIFO_DOUT_OUT,
			FIFO_EMPTY				=> FIFO_EMPTY_OUT,
			
			Reset 					=> Reset_Inverted
		);


        USB_PYH_Instance : entity work.USB_PHY 
        port MAP(
            USB_DATA                => USB_DATA,
            -- When high, do not read data
            USB_NOT_RXF             => USB_NOT_RXF,
            -- When high, do not write data 
            USB_NOT_TXE             => USB_NOT_TXE,
            -- When low, data is read 
            USB_NOT_RD              => USB_NOT_RD,
            -- When low, data is written 
            USB_NOT_WR              => USB_NOT_WR,

            DATA_IN_DATA            => DATA_IN_DATA,
            DATA_IN_VALID           => DATA_IN_VALID,
            DATA_IN_NEXT            => DATA_IN_NEXT,
            DATA_IN_STARTED         => DATA_IN_STARTED,

            DATA_OUT_DATA           => DATA_OUT_DATA,
            DATA_OUT_VALID          => DATA_OUT_VALID,
            DATA_OUT_NEXT           => DATA_OUT_NEXT,
            DATA_OUT_STARTED        => DATA_OUT_STARTED,

            Clk                     => Clk,
            Reset                   => Reset_Inverted
    );

end behavioral;

