library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity FIFO_UUT is
end FIFO_UUT;

architecture BEHAVIOURAL of FIFO_UUT is

          signal DATA   : std_logic_vector(8 downto 0) := (others => '0');
          signal Q      : std_logic_vector(8 downto 0) := (others => '0');
          signal WE     : std_logic := '0';
          -- signal RE     : std_logic := '0';
          signal FULL   : std_logic := '0';
          signal EMPTY  : std_logic := '0';
          signal RESET  : std_logic := '0';
          signal AEMPTY : std_logic := '0';
          signal AFULL  : std_logic := '0';

          signal Clk    : std_logic := '0';
          signal Counter : unsigned(10 downto 0) := (others=>'0'); 
          signal MasterCounter : unsigned(127 downto 0) := (others=>'0'); 

          -- FIFO FALL THROUGH LOGIC 
          signal dout_valid : std_logic := '0';
          signal fifo_rd_en : std_logic := '0';
          signal fifo_empty : std_logic := '0';
          signal rd_en : std_logic := '0';

          signal TestX  : std_logic_vector(8 downto 0) := (others => '0');

begin

    MyFIFO : entity work.FIFO_DISASSEMBLE
    port MAP( 
        DATA => DATA,
        Q    => Q,
        WE   => WE,
        RE   => fifo_rd_en,
        WCLOCK => clk,
        RCLOCK => clk,
        FULL => FULL,
        EMPTY => fifo_empty,
        RESET => RESET,
        AEMPTY => AEMPTY,
        AFULL => AFULL
        );

 TheMasterClock : process
begin
	clockloop: for i in 0 to 25000 loop
		clk <= NOT clk;
		wait for 20ns;
	end loop clockloop;
	wait;
end process TheMasterClock; 

Stimulus : process(clk) 
begin
	if rising_edge(clk) then
		

		MasterCounter <= MasterCounter + 1;
        WE <= '0';
        rd_en <= '0';
        DATA <= (others => '0');
        TestX   <= (others => '0');

		
		if MasterCounter < to_unsigned(5,128) then
			Reset <= '1';
		else
			Reset <= '0';
		end if;

		if MasterCounter = to_unsigned(15,128) then
            WE <= '1';
            DATA <= "111111111";
		elsif MasterCounter = to_unsigned(16,128) then
            WE <= '1';
            DATA <= "111110000";
        --elsif MasterCounter = to_unsigned(25,128) then
        --    rd_en <= '1';
		else
            null;
		end if;
        
        if (dout_valid = '1') AND (rd_en = '0') then
            rd_en <= '1';  
            TestX <= Q;
        else
            null;
        end if;

	end if;
end process Stimulus;

    -- Synchronous FIFO for SmartFusion, IGLOO, ProASIC3 and Fusion Summary
    -- PIPLINE: NO
    -- Reset: Active High
    -- Write Enable: Active High
    -- Read Enable: Active High 
    fifo_rd_en <= (NOT fifo_empty) AND ((NOT dout_valid ) OR rd_en);
    FIFOFALLTHROUGH : process (Reset,Clk)
    begin
        if Reset = '1' then

            dout_valid <= '0';

        elsif rising_edge(Clk) then

            dout_valid <= dout_valid;
            
            if fifo_rd_en = '1' then
                dout_valid <= '1';
            elsif rd_en = '1' then
                dout_valid <= '0';
            else
                null;
            end if;

        end if;

    end process FIFOFALLTHROUGH;

end;
